/******************************************************************************
*
*   PROJECT NAME        :   UGLD
*   FILE NAME           :   D2830101.DDL
*   START DATE          :   02-12-2018
*   DESCRIPTION         :   UGLD HART 7 DD.
*                           
*   ENVIRONMENT         :   DD-IDE 5.0
*                           
*   AUTHOR              :   Harinath Babu
*                           
*   MANUFACTURER's ID   :   210 (0xD2)
*   TRANSMITTER HART    :   07  (0x07)
*   TRANSMITTER Type    :   131 (0x83)
*   DEVICE REVISION     :   01  (0x01)
*   
*   COMMENTS            :   UGLD GAS SENSOR DEVICE DESCRIPTION FILE
*******************************************************************************
*
*               COPYRIGHT (c) 2018
*               HONEYWELL INC.,
*               ALL RIGHTS RESERVED
*
*       This software is a copyrighted work and/or information protected as
*       a trade secret. Legal rights of Honeywell Inc. in this software is
*       distinct from ownership of any medium in which the software is
*       embodied. Copyright or trade secret notices included must be
*       reproduced in any copies authorized by Honeywell Inc.  The
*       information in this software is subject to change without notice and
*       should not be considered as a commitment by Honeywell Inc.
*
*******************************************************************************

********************* Version and Modification History ************************
Version : 1.0  02/12/2017 Harinath  Initial version

*/

MANUFACTURER HONEYWELL_ANALYTICS, DEVICE_TYPE _UGLD, DEVICE_REVISION 1, DD_REVISION 43
/*****************************************************************************
* Include Files
******************************************************************************
*/
#include "macros.h"
#include "methods.h"
#include "ddstrings.h"

#define LEVEL0  0
#define LEVEL1  1
#define LEVEL2  2
#define UPDATE_IN_PROGRESS      8
#define DEVICE_BUSY             32
#define MAXIMUM_RESET_RETRIES   75
#define CALIBRATION_TIMEOUT     50

/**************************************************************
 *                 Import standard tables                     *
 **************************************************************/

IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 4 
{

    EVERYTHING;

    REDEFINITIONS
    {
        VARIABLE loop_current_mode
        {
            REDEFINE LABEL "|en|Loop Current Mode";
            REDEFINE CLASS DEVICE & DYNAMIC;
            REDEFINE TYPE ENUMERATED
            {
               {0 , "|en|Multi drop" , "|en|Fixed current mode"}, 
               {1 , "|en|Point to point", "|en|Point to point current mode"}
            }
        }  
    }
}
/**************************************************************
*       Import Universal Command Specifications               *
***************************************************************/

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 2
{
    EVERYTHING;
  
    REDEFINITIONS
    {
        VARIABLE device_type
        {
            REDEFINE TYPE ENUMERATED(2)
            {
                { 0xD283,  "|en|Searchzone Sonik" }
            }
        }
    
        VARIABLE manufacturer_id
        {
            REDEFINE TYPE ENUMERATED(2)
            {   
                { 0x00D2,  "|en|Honeywell Analytics" }    
            }
        }
        
        VARIABLE private_label_distributor 
        {
            REDEFINE TYPE ENUMERATED(2)  
            {
                {0x00D2, "|en|Honeywell Analytics"}
            }
        }

        VARIABLE response_preambles
        {
            REDEFINE HELP "Number of Response Preambles- Number of preambles to be sent "
                "in the response message from the Field Device to the Host."
                " Minimum is 5 and Maximum is 20 ";
        }

        COMMAND read_dynamic_variables_and_pv_current 
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DAQ.ANALOG_VALUE,
                    PV.DEVICE_VARIABLE.DIGITAL_UNITS, 
                    PV.DEVICE_VARIABLE.DIGITAL_VALUE                    
                }
            }
        }

        /* The transaction will need to be redefined to show the correct
            number of dynamic variables returned in Command 8 */
        COMMAND read_dynamic_variable_classification   
        {
            REDEFINE TRANSACTION 
            {
                REQUEST {}
                REPLY 
                {
                    response_code, device_status,
                    PV.DEVICE_VARIABLE.CLASSIFICATION                   
                }
            }
        }

        COMMAND read_additional_device_status   /* Command 48 */
        {
            REDEFINE TRANSACTION 0
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0, device_specific_status_1, device_specific_status_2,
                    device_specific_status_3, device_specific_status_4, device_specific_status_5,
                    extended_fld_device_status, 0x00,
                    standardized_status_0                   
                }
            }
            REDEFINE TRANSACTION 1
            {
                REQUEST 
                {
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),
                    extended_fld_device_status (INFO), 0x00,
                    standardized_status_0 (INFO)
                }
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),
                    extended_fld_device_status (INFO), 0x00,
                    standardized_status_0 (INFO)                    
                }
            }
        }

        VARIABLE device_specific_status_0 /* Command 48 */ 
        {
            REDEFINE LABEL "Active Faults[0-7]";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { BIT_0, "Operating State fault",            HARDWARE },  
                { BIT_1, "Temperature fault",                HARDWARE },
                { BIT_2, "Bad 24V Supply",                   HARDWARE },
                { BIT_3, "Bad 5V Supply",                    HARDWARE },
                { BIT_4, "4-20mA Read back error",           HARDWARE },
                { BIT_5, "4-20mA Feedback supply failure",   HARDWARE },
                { BIT_6, "4-20mA wiring fault",              HARDWARE },
                { BIT_7, "4-20mA daignostic error",          HARDWARE }
            }
        }

        VARIABLE device_specific_status_1  /* command 48 */
        {
            REDEFINE LABEL "Active Faults[8-15]";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { BIT_0, "Relay 1 Current monitoring failure",                        HARDWARE },     
                { BIT_1, "Relay 1 pulse diagnsotics failure",                         HARDWARE },
                { BIT_2, "Relay 1 power diagnsotics failure",                         HARDWARE },
                { BIT_3, "Relay 2 Current monitoring failure",                        HARDWARE },
                { BIT_4, "Relay 2 pulse diagnsotics failure",                         HARDWARE },
                { BIT_5, "Relay 2 power diagnsotics failure",                         HARDWARE },
                { BIT_6, "Relay 3 Current monitoring failure",                        HARDWARE },
                { BIT_7, "Relay 3 pulse diagnsotics failure",                         HARDWARE }
            }
        }
               
        VARIABLE device_specific_status_2 /* command 48 */
        {
            REDEFINE LABEL "Active Faults[16-23]";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { BIT_0, "Relay 3 power diagnsotics failure",                      HARDWARE },
                { BIT_1, "Watchdog diagnostics failure",                           SOFTWARE },
                { BIT_2, "AFE diagnostics failure",                                SOFTWARE },
                { BIT_3, "Reserved",                                               SOFTWARE },
                { BIT_4, "Software diagnostics failure",                           SOFTWARE },
                { BIT_5, "Wireless fault",                                         HARDWARE },
                { BIT_6, "Power On diagnostics failure",                           SOFTWARE },
                { BIT_7, "Reserved",                                               HARDWARE }
            }
        }         

        VARIABLE device_specific_status_3  /* command 48 */
        {
            REDEFINE LABEL "Active Faults[24-31]";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { BIT_0, "Reserved",                       HARDWARE },
                { BIT_1, "Reserved",                       HARDWARE },
                { BIT_2, "Reserved",                       SOFTWARE },
                { BIT_3, "Reserved",                       SOFTWARE },
                { BIT_4, "Reserved",                       SOFTWARE },
                { BIT_5, "Reserved",                       SOFTWARE },
                { BIT_6, "Reserved",                       SOFTWARE },
                { BIT_7, "Reserved",                       SOFTWARE }
            }
        }     

        VARIABLE device_specific_status_4 /* command 48 */
        {  
            REDEFINE LABEL "Active Faults[32-39]";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { BIT_0, "Reserved",                       HARDWARE },
                { BIT_1, "Reserved",                       HARDWARE },
                { BIT_2, "Reserved",                       HARDWARE },
                { BIT_3, "Reserved",                       HARDWARE },
                { BIT_4, "Reserved",                       HARDWARE },
                { BIT_5, "Reserved",                       HARDWARE },
                { BIT_6, "Reserved",                       HARDWARE },
                { BIT_7, "Reserved",                       HARDWARE }
            }
        }      

        VARIABLE device_specific_status_5  /* command 48 */
        {
            REDEFINE LABEL "Active Faults[40-48]"; 
            REDEFINE TYPE BIT_ENUMERATED
            {
                { BIT_0, "Reserved",                        HARDWARE },
                { BIT_1, "Reserved",                        HARDWARE },
                { BIT_2, "Reserved",                        HARDWARE },
                { BIT_3, "Reserved",                        HARDWARE },
                { BIT_4, "Reserved",                        SOFTWARE },
                { BIT_5, "Reserved",                        SOFTWARE },
                { BIT_6, "Reserved",                        SOFTWARE },
                { BIT_7, "Reserved",                        SOFTWARE }
            }
        }      
    }
}

IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
{
    EVERYTHING;

    REDEFINITIONS
    {
        /*
         *  Remove the device variables not used in this DD. 
         */
        DELETE COLLECTION secondary_variable;
        DELETE COLLECTION tertiary_variable;
        DELETE COLLECTION quaternary_variable;

        ARRAY OF COLLECTION dynamic_variables
        {
            ELEMENTS
            {
                DELETE SECONDARY;
                DELETE TERTIARY;
                DELETE QUATERNARY;
            }
        }
        COLLECTION OF VARIABLE analog_io
        {
            
            MEMBERS
            {
                /* Analog Information */
               REDEFINE ANALOG_VALUE,           conc_analog_value,  [loop_current],[analog_value_help];
               REDEFINE ALARM_CODE,             loop_alarm_code,    [alarm_type],     [alarm_code_help];
               REDEFINE ANALOG_CHANNEL_FLAGS,   loop_flags,         [analog_channel_flags],[analog_channel_flags_help];
            }
        }
        /* 
        *  Most devices use the same data item for range units (command 15) and 
        *  the sensor units. when this is true the following should be added.  
        *  If your device uses a different variable for range units you 
        *  should reference it here 
        */
        COLLECTION OF VARIABLE scaling
        {
            MEMBERS
            {
                ADD RANGE_UNITS, deviceVariables[0].DIGITAL_UNITS, [range_units], [range_units_help];
            } 
        }
        /* Upper range value of Gas concentraiton */
        VARIABLE upperRange_value
        {
            REDEFINE HELP "Full Scale Value";
        }
        /* Lower range value of Gas concentraiton */
        VARIABLE lowerRange_value
        {
            REDEFINE HELP "Lower Range Value,4 mA Equivalent Gas Present";
        }
        /* Percentage range of gas concentration */
        VARIABLE percentRange                    
        {
            REDEFINE HELP " % of range";
        }
        
        VARIABLE loop_alarm_code
        {
            REDEFINE LABEL "|en|PV Alarm Status";
            REDEFINE CLASS ANALOG_CHANNEL & DYNAMIC;            
            REDEFINE HELP  "|en|Alarm Status of the Device";
            REDEFINE TYPE ENUMERATED
            {
                 {251 , "|en|None"},
                 {241 , "|en|ALARM ACTIVE"}
            }
        }
        /* Configuration change counter */
        VARIABLE config_change_counter
        {
            REDEFINE LABEL  "|en|Config Revision";
        }
        VARIABLE longTag
        {
           REDEFINE LABEL  "|en|Long Tag";
           REDEFINE HELP   "|en|32 Character Transmitter Name used for Identification Purpose";
        }
        VARIABLE polling_address
        {
           REDEFINE LABEL  "|en|Poll Address"; 
           REDEFINE CLASS DEVICE & DYNAMIC;
        }
        VARIABLE message
        {
            REDEFINE LABEL  "|en|Calibration Info"; 
            REDEFINE HELP   "|en|Use to log calibration details like engineer's initials and date";
        }
        VARIABLE descriptor
        {
            REDEFINE LABEL "|en|Description";
        }
        VARIABLE date
        {
            REDEFINE LABEL  "|en|Assembly Date";  
        }
        VARIABLE final_assembly_number
        {
            REDEFINE LABEL  "|en|Assembly Number";
        }
        VARIABLE tag
        {
            REDEFINE LABEL  "|en|Short Tag";
            REDEFINE HELP   "|en|Short Tag used for Identification Purpose";
        } 
    }
}

/**************************************************************
*       Import Common Practice Command Specifications         *
***************************************************************/

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 2
{
    VARIABLE device_variable_trim_code;

}
/*****************************************************************************
* Declare mandatory and other stuff required for universal commands
*
* Declare primary variables and structures 
******************************************************************************
*/

/* Primar variable (Gas) of the Measurement Units */
VARIABLE conc_unit
{
    HELP [digital_units_help];
    CLASS CORRECTION;
    LABEL "Concentration Unit";
    HANDLING READ;    
    TYPE ENUMERATED 
    {
        { 0x9C,   "dB" }                                             /* decibel */ 
    }
    DEFAULT_VALUE 0x9C;
}

/* Primary variable value (Gas) */
VARIABLE conc_value
{
    LABEL "|en|Ultrasonic Level";
    HELP "|en|Displays current Ultrasonic Sound Pressure";
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

/* Loop current in mA */
VARIABLE conc_analog_value
{
    LABEL "|en|mA Loop O/P";
    HELP "|en|Gas Concentration forced in mA Loop Output";
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
    CONSTANT_UNIT "|en|mA";
}

/* not used */
/* Note: future purpose */
VARIABLE concentrationDQ 
{
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED 
    {
        {0 , "|en|Bad"},  
        {1 , "|en|Manual / Fixed"},
        {2 , "|en|Poor Accuracy"},
        {3 , "|en|Good"} 
    }
}

/* not used */
/* Note: future purpose */
VARIABLE concentrationLS 
{
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED (2) 
    {
        {0 , "|en|Not Limited"},  
        {1 , "|en|Low Limited"},
        {2 , "|en|High Limited"},
        {3 , "|en|Constant"}
    }
}

/* not used */
/* Note: future purpose */
VARIABLE concentrationStatus 
{
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE BIT_ENUMERATED 
    {
        {BIT_7 , "More Device Family Status Available",  MORE}
    } 
}

/* not used */
/* Note: future purpose */
VARIABLE concClassificationCode 
{
    LABEL   "|en|Measurement Type";
    HANDLING READ;
    TYPE ENUMERATED 
    {
        {   0,  "|en|Not Classified"}, 
        {   90, "|en|Concentration"}
    }
}

/* not used */
/* Note: future purpose */
VARIABLE conc_analog_unit
{
    HELP [analog_value_unit_help];
    CLASS ANALOG_OUTPUT;
    LABEL "|en|Analog Channel Unit";
    HANDLING READ;
    TYPE ENUMERATED 
    {
        {39, [mA], [milliamperes_help]}
    }  
}

VARIABLE conc_upper_limit   /* Command 14 */
{
    HELP "|en|Upper Limit Value: Defines the maximum usable value of Concentration.";
    CLASS CORRECTION;
    LABEL "|en|USL";
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE conc_lower_limit   /* Command 14 */
{
    HELP "|en|Lower Limit Value: Defines the minimum usable value of Concentration.";
    CLASS CORRECTION;
    LABEL "|en|LSL";
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE conc_min_span
{
    HELP [minimum_span_help];
    CLASS CORRECTION;
    LABEL "Min Span";
    CONSTANT_UNIT [Percent];
    HANDLING READ&WRITE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE conc_damping
{
    HELP [seconds_damping_value_help];
    CLASS DEVICE;
    LABEL "PV Damp";
    CONSTANT_UNIT [sec];    
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE conc_serial_number
{
    HELP "Gas Sensor Serial Number: Unique identifier for ONG's Gas sensor.";
    CLASS DEVICE;
    LABEL "Sensor S/N"; 
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(3);
}

/* Unit Relation */
UNIT pv_units_relations
{
    conc_unit:
        PV.RANGING.LOWER_RANGE_VALUE,
        PV.RANGING.UPPER_RANGE_VALUE,
        conc_upper_limit,
        conc_lower_limit,
        conc_value
        
}

VARIABLE min_zero
{
    LABEL "|en|Min Zero";
    CLASS DEVICE;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE max_zero
{
    LABEL "|en|Max Zero";
    CLASS DEVICE;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE min_span
{
    LABEL "|en|Min Span";
    CLASS DEVICE;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE max_span
{
    LABEL "|en|Max Span";
    CLASS DEVICE;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE min_diff
{
    LABEL "|en|Min differential";
    CLASS DEVICE;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE last_zero
{
    LABEL "|en|Last Zero";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE last_span
{
    LABEL "|en|Last Span";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }
}

VARIABLE cal_point_codes
{
    CLASS LOCAL;
    HANDLING READ;
    TYPE ENUMERATED
    {
        /* 0 is reserved */
        { 1, [lower_trim_point_supported] },  
        { 2, [upper_trim_point_supported] },  
        { 3, [lower_and_upper_trim_point_supported] }
    }
}

COLLECTION OF VARIABLE concentration
{
    LABEL "|en|ONG - IR Sensor";
    HELP [primary_variable_help];
    MEMBERS
    {
        /* Sensor Correction Information */
        DIGITAL_UNITS,          conc_unit;
        DIGITAL_VALUE,          conc_value;
        DATA_QUALITY,           concentrationDQ;
        LIMIT_STATUS,           concentrationLS;
        DEVICE_FAMILY_STATUS,   concentrationStatus;
        CLASSIFICATION,         concClassificationCode;

        UPPER_RANGE_VALUE,      upperRange_value,   [urv],                  [upper_range_value_help];
        LOWER_RANGE_VALUE,      lowerRange_value,   [lrv],                  [lower_range_value_help];

        UPPER_SENSOR_LIMIT,     conc_upper_limit;
        LOWER_SENSOR_LIMIT,     conc_lower_limit;
        MINIMUM_SPAN,           conc_min_span;
        DAMPING_VALUE,          conc_damping; 
        ALARM_CODE,             loop_alarm_code, [alarm_select], [alarm_code_help];
        SENSOR_SERIAL_NUMBER,   conc_serial_number, "S/n";

        /* Trim Procedure */
        TRIM_POINT_SUPPORT,         cal_point_codes ;
        TRIM_POINT_UNITS ,          conc_unit;
        MINIMUM_LOWER_TRIM_POINT,   min_zero;  
        MAXIMUM_LOWER_TRIM_POINT,   max_zero;
        MINIMUM_UPPER_TRIM_POINT,   min_span;
        MAXIMUM_UPPER_TRIM_POINT,   max_span;
        MINIMUM_TRIM_DIFFERENTIAL,  min_diff;
        LOWER_TRIM_POINT,           last_zero;
        UPPER_TRIM_POINT,           last_span;
    }
}

ARRAY OF COLLECTION deviceVariables 
{
    ELEMENTS
    {
        0, concentration,   [pv];        
    }
}

/* Used by HART Communicator for displaying a loop warning when writing */
ARRAY OF VARIABLE loop_warning_variables
{
    ELEMENTS
    {
        1, PV.RANGING.LOWER_RANGE_VALUE, [lrv], [lower_range_value_help];
        2, PV.RANGING.UPPER_RANGE_VALUE, [urv], [upper_range_value_help];
        3, loop_current_mode;  
    }
}

/******END Re declaration of variable requqired UNIV commands and CPC commands*******************/

/******************************************************************************************
 ********************** MENUS SECTION *****************************************************
 ******************************************************************************************/

/*--------------------- ROOT menu ----------------------------*/
MENU root_menu
{
    LABEL  "Main menu";
    ITEMS
    {   
         login_level(DISPLAY_VALUE,READ_ONLY),
         mth_get_pw,
         mnu_DeviceOverview,
         mnu_DeviceSetup
    }
}

MENU mnu_DeviceOverview
{
    LABEL "Device Overview";
    HELP "|en|This menu contains key device parameters for visualization";
    ITEMS
    {
         BT_DeviceName_Rd(DISPLAY_VALUE,READ_ONLY),
         Current_OpMode(DISPLAY_VALUE,READ_ONLY),
         conc_value(DISPLAY_VALUE,READ_ONLY),
         PV.DAQ.ANALOG_VALUE(DISPLAY_VALUE,READ_ONLY),
         Loop_OpMode_Rd(DISPLAY_VALUE,READ_ONLY),
         Algorithm_Mode_Rd(DISPLAY_VALUE,READ_ONLY),
         ifunctiontestdue(DISPLAY_VALUE, READ_ONLY),
         RTC_Time(DISPLAY_VALUE,READ_ONLY),
         RTC_Date(DISPLAY_VALUE,READ_ONLY)
    }
}

MENU mnu_DeviceSetup
{
    LABEL "Device Setup";
    HELP "|en|This contains all device configuration menus.Available with Engineer or Honeywell Engineer login";
    ITEMS
    {
        mnu_operations,
        mnu_diagnostics,
        evtView,
        mnu_settings,
        mnu_account,
        mnu_FieldEngData
    }
}

/*-------------------------------Sub Menu 1 Screens----------------------------*/
MENU mnu_operations
{
    LABEL "|en|Operations";
    HELP "|en|Menu to change device state and test";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Current_OpMode(DISPLAY_VALUE,READ_ONLY),
        mnu_inhibit,
        mnu_Test,
        mth_ClrFlts
    }
}

MENU mnu_diagnostics 
{
    LABEL "|en|Diagnostics";
    HELP  "|en|Displays the faults and warnings present on the device";
    #if __TOKVER__ >= 602
    STYLE PAGE;
    #endif
    ITEMS
    {
        evtView_warnings,
        evtView_faults
    }
}

MENU mnu_settings
{
    LABEL "|en|Settings";
    HELP "|en|Menu to edit device configuration parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Current_OpMode(DISPLAY_VALUE,READ_ONLY),
        mth_wr_inhibit,
        mnu_algorithm_mode,
        mnu_alarms,
        mnu_loop,
        mnu_led,
        mnu_relay,
        mnu_BT,
        mnu_functiontest,
        mnu_HART,
        mnuAssemblyInfo
    }
}

MENU mnu_account
{
    LABEL "|en|Account";
    HELP "|en|Menu to change password, date and time";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        mth_set_pw,
        mth_change_TIME,
        mth_change_DATE
    }
}

/*-------------------------------Sub Menu 2 Screens----------------------------*/
MENU mnu_inhibit
{
    LABEL "|en|Inhibit Detector";
    HELP "|en|Menu to edit inhibit parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        inhibit_mode(DISPLAY_VALUE,READ_ONLY),
        Inhibit_Timeout(DISPLAY_VALUE,READ_ONLY),
        mth_start_inhibit,
        mth_end_inhibit,
        mnu_wr_inhibit_timeout
    }
}

MENU mnu_Test
{
    LABEL "|en|Output Test";
    HELP "|en|Menu to test device out put interfaces.";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
         Force_Timeout(DISPLAY_VALUE,READ_ONLY),
         mth_mA_force,
         mth_relay_test,
         mth_led_test,
         mnu_wr_force_timeout
    }
}

MENU mnu_algorithm_mode
{
    LABEL "|en|Algorithm Mode";
    HELP "|en|Menu to edit algorithm mode parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
         Algorithm_Mode_Rd(DISPLAY_VALUE,READ_ONLY),
         mnu_configure_algorithm_mode
    }
}

MENU mnu_alarms
{
    LABEL "Alarms";
    HELP "|en|Configure the internal alarm threshold for the detector";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        alarm_set_point(DISPLAY_VALUE,READ_ONLY),
        alarm1_mA(DISPLAY_VALUE,READ_ONLY),
        alarm2_mA(DISPLAY_VALUE,READ_ONLY),
        alarm_delay(DISPLAY_VALUE,READ_ONLY),
        alarm_latch_st(DISPLAY_VALUE,READ_ONLY),
        mnu_configureAlarms
    }
}

MENU mnu_loop
{
    LABEL "mA Loop";
    HELP "|en|Configurable mA loop parameters";
    VALIDITY IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Loop_OpMode_Rd(DISPLAY_VALUE,READ_ONLY),
        FaultCurrent(DISPLAY_VALUE,READ_ONLY),
        InhibitCurrent(DISPLAY_VALUE,READ_ONLY)
        WarningCurrent(DISPLAY_VALUE,READ_ONLY),
        OverrangeCurrent(DISPLAY_VALUE,READ_ONLY),
        mnu_loopconfig
    }
}

MENU mnu_led
{
    LABEL "LED";
    HELP "|en|Configurable LED parameters";
    VALIDITY IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        LEDIntensity_Rd(DISPLAY_VALUE,READ_ONLY),
        LEDMode_Rd(DISPLAY_VALUE,READ_ONLY),
        mnu_ledconfig
    }
}

MENU mnu_relay
{
    LABEL "Relays";
    HELP "|en|Configurable Relay parameters";
    VALIDITY IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Relay1(DISPLAY_VALUE,READ_ONLY),
        Relay1_latch(DISPLAY_VALUE,READ_ONLY),
        Relay2(DISPLAY_VALUE,READ_ONLY),
        Relay2_latch(DISPLAY_VALUE,READ_ONLY),
        Relay3(DISPLAY_VALUE,READ_ONLY),
        Relay3_latch(DISPLAY_VALUE,READ_ONLY),
        mnu_relayconfig
    }
}

MENU mnu_BT
{
    LABEL "Bluetooth";
    HELP "|en|Menu to edit Bluetooth interface parameters";
    VALIDITY IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        BT_Status(DISPLAY_VALUE,READ_ONLY),
        BT_Address(DISPLAY_VALUE,READ_ONLY),
        BT_DeviceName_Rd(DISPLAY_VALUE,READ_ONLY),
        BT_AdvIntval(DISPLAY_VALUE,READ_ONLY),
        mnu_BTconfig
    }
}

MENU mnu_functiontest
{
    LABEL "Function Test Period";
    HELP "|en|Functiona Test interval configuration";
    VALIDITY IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Function_Test_Interval(DISPLAY_VALUE,READ_ONLY),
        mnu_functiontestconfig
    }
}

MENU mnu_HART
{
    LABEL "HART";
    HELP "|en|Configurable HART parameters";
    VALIDITY IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        polling_address(DISPLAY_VALUE,READ_ONLY), 
        tag(DISPLAY_VALUE,READ_ONLY),
        longTag(DISPLAY_VALUE,READ_ONLY),
        descriptor(DISPLAY_VALUE,READ_ONLY),
        loop_current_mode(DISPLAY_VALUE,READ_ONLY),
        mnu_HARTconfig,
        mnu_hart_param
    }
}

MENU mnu_FieldEngData
{   LABEL "Field Engineer Data";
    HELP "Field Engineer Data";
    VALIDITY IF (login_level == LEVEL2){TRUE;}ELSE{FALSE;}    
    ITEMS
    {
        fTemperature(DISPLAY_VALUE, READ_ONLY),
        fTargetCurrent(DISPLAY_VALUE, READ_ONLY),
        fMeasuredCurrent(DISPLAY_VALUE, READ_ONLY),
        f5vSupply(DISPLAY_VALUE, READ_ONLY),
        fSupplyVoltage(DISPLAY_VALUE, READ_ONLY),
        ifunctiontestdue(DISPLAY_VALUE, READ_ONLY),
        relay1state(DISPLAY_VALUE, READ_ONLY),
        relay2state(DISPLAY_VALUE, READ_ONLY),
        relay3state(DISPLAY_VALUE, READ_ONLY)
    }
    STYLE TABLE;
}

MENU mnuAssemblyInfo
{
    LABEL "Assembly Details";
    HELP "|en|Factory setting parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        config_change_counter(DISPLAY_VALUE,READ_ONLY),
        polling_address(DISPLAY_VALUE,READ_ONLY), 
        descriptor(DISPLAY_VALUE,READ_ONLY),
        date(DISPLAY_VALUE,READ_ONLY),
        final_assembly_number(DISPLAY_VALUE,READ_ONLY),
        tag(DISPLAY_VALUE,READ_ONLY),
        longTag(DISPLAY_VALUE,READ_ONLY),
        device_id(DISPLAY_VALUE,READ_ONLY)
    }
}

MENU mnu_hart_param
{
    LABEL "HART Parameters";
    HELP "|en|Device HART interface parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        loop_current_mode(DISPLAY_VALUE,READ_ONLY),
        request_preambles(DISPLAY_VALUE,READ_ONLY),
        response_preambles(DISPLAY_VALUE,READ_ONLY),
        device_id(DISPLAY_VALUE,READ_ONLY), 
        universal_revision(DISPLAY_VALUE,READ_ONLY),
        transmitter_revision(DISPLAY_VALUE,READ_ONLY),
        software_revision(DISPLAY_VALUE,READ_ONLY),
        hardware_revision(DISPLAY_VALUE,READ_ONLY)
    }
}

/*-------------------------------Sub Menu 3 Screens----------------------------*/
MENU mnu_wr_inhibit_timeout
{
    LABEL "Edit Inhibit Timeout";
    HELP "|en|Facility to edit Inhibit Timeout\n\n '0' means no timeout \n\n Range: 60 - 600Sec";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Inhibit_Timeout
    }
}

MENU mnu_configure_algorithm_mode
{
    LABEL "|en|Change Algo Mode";
    HELP "|en|Edit the Algorithm mode to the following\n\n"
         "|en|Standard\n"
         "|en|Focused";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
         Algorithm_Mode
    }
}

MENU mnu_configureAlarms
{
    LABEL "Configure Alarms";
    HELP "|en|Configure the internal alarm threshold for the detector";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        alarm_set_point,
        alarm1_mA,
        alarm2_mA,
        alarm_delay,
        mth_config_latch
    }
}

MENU mnu_wr_force_timeout
{
    LABEL "Edit Force Timeout";
    HELP "|en|Facility to edit Force Timeout\n\n '0' means no timeout\n\n Range: 0 - 600Sec";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Force_Timeout
    }
}

MENU mnu_loopconfig
{
    LABEL "Configure mA Loop";
    HELP "|en|Configuralble mA loop parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        Loop_OpMode,
        FaultCurrent,
        InhibitCurrent,
        WarningCurrent,
        OverrangeCurrent
    }
}

MENU mnu_ledconfig
{
    LABEL "Configure LED";
    HELP "|en|Configurable LED parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        LEDIntensity,
        LEDMode
    }
}

MENU mnu_relayconfig
{
    LABEL "Change Relays Status";
    HELP "|en|Relay configuration";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        mth_Relay1,
        mth_Relay1_latch,
        mth_Relay2,
        mth_Relay2_latch,
        mth_Relay3,
        mth_Relay3_latch
    }
}

MENU mnu_BTconfig
{
    LABEL "Configure Bluetooth";
    HELP "|en|Configurable Bluetooth parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        BT_Status,
        BT_DeviceName,
        BT_AdvIntval,
        mth_BT_newPIN
    }
}

MENU mnu_functiontestconfig
{
    LABEL "Change Interval";
    HELP "|en|Facility to edit function test interval";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {   
        Function_Test_Interval
    }
}

MENU mnu_HARTconfig
{
    LABEL "Configure HART";
    HELP "|en|Facility to edit HART configurable parameters";
    #if __TOKVER__ >= 602
    STYLE DIALOG;
    #endif
    VALIDITY 
    IF (login_level > LEVEL0) {TRUE;} ELSE {FALSE;}
    ITEMS
    {
        mth_polling_address, 
        tag,
        longTag,
        descriptor,
        mth_loop_current_mode
    }
}

/******************************************************************************************
 ********************** END: MENUS SECTION ************************************************
 ******************************************************************************************/

/******************************************************************************************
*       Implimentation of Device Specific commands                                        *
*******************************************************************************************/
METHOD mth_loop_current_mode
{
    LABEL "Loop Current Mode";
    HELP  [loop_current_mode_help];
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
		    strPrompt = "|en|Select loop current mode to";
			GET_DEV_VAR_VALUE(strPrompt, loop_current_mode);			
            result = send(6,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure loop current mode Successful";
            }
            else
            {
                strPrompt = "|en|Configure loop current mode NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_polling_address
{
    LABEL "|en|Poll Address";
    HELP  [polling_address_help];
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
		    strPrompt = "|en|Configure Poll Address to";
			GET_DEV_VAR_VALUE(strPrompt, polling_address);			
            result = send(6,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure Poll Address Successful";
            }
            else
            {
                strPrompt = "|en|Configure Poll Address NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

/*------------------- COMMAND 128 Read fault number  ----------------*/
VARIABLE iFaultNumber
{
    LABEL "Fault Number";
    HELP  "";
    CLASS DEVICE;
    HANDLING READ;
    TYPE INTEGER;
}

COMMAND CMD_READ_FAULT_NUMBER
{
    NUMBER 128;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,iFaultNumber
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 129  Read current operating mode----------------*/
VARIABLE Current_OpMode
{
    LABEL "Detector Status";
    HELP "Field Specific Operating Mode which can contain any one of the following status\n\n"
                          "PowerOn\n"
                          "Fault\n" 
                          "Inhibit\n"
                          "Healthy\n"
                          "Pre Alarm\n"
                          "Alarm\n"
                          "Warning\n"
                          "Overrange\n";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED
            {
                {0,"|en|PowerOn"}
                {1,"|en|Fault"}      /* Non recoverable fault */
                {2,"|en|Inhibit"}
                {3,"|en|Alarm"}
                {4,"|en|Fault"}
                {5,"|en|Warning"}
                {6,"|en|Healthy"}
                {0x10,"|en|Pre Alarm"}
                {0x11,"|en|Alarm"}
                {0x12,"|en|Over range"}
            }
}

COMMAND CMD_READ_OPMODE
{
    NUMBER 129;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, Current_OpMode
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 130  Read RTC (UTC)----------------*/
VARIABLE RTC_Time
{
    LABEL "Time (24-hour)";
    HELP "|en|Field Device Time in 24-hour format"; 
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    #if __TOKVER__ >= 602
    #endif
    TYPE ASCII(5);
    DEFAULT_VALUE "16:14";
}

VARIABLE RTC_Date
{
    LABEL "Date (dd/mm/yyyy)";
    HELP "|en|Field Device Date in dd/mm/yyyy format";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ASCII(10);
    DEFAULT_VALUE "09/02/2018";
}

VARIABLE UTC_Date
{
    LABEL "UTC";
    HELP "|en|Field Device Date";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(4);
    DEFAULT_VALUE 0;
}

COMMAND CMD_READ_RTC
{
    NUMBER 130;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, 
            RTC_Date, 
            RTC_Time
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 131 Write RTC ----------------*/
VARIABLE Change_Date_Time_Sel
{
    LABEL "Change Date Time Sel";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE INTEGER;
}

VARIABLE Change_RTC_Time
{
    LABEL "Set Time(24-hour)";
    HELP  "|en|Detector time configuration using 24 hours format (hh.mm)";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ASCII(5);

    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE "16:14";
}

METHOD mth_change_TIME
{
    LABEL "Set Time(24-hour)";
    HELP  "|en|Detector time configuration using 24 hours format (hh.mm)";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        add_abort_method(user_abort);
        iassign(Change_Date_Time_Sel, 1);
        vassign(Change_RTC_Time, RTC_Time);
        vassign(Change_RTC_Date, RTC_Date);
        strPrompt = "Set Time(24-hour)hh:mm";
        GET_DEV_VAR_VALUE(strPrompt, Change_RTC_Time);
        result = send(131,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   
            strPrompt = "|en|Time set in progress";
            DELAY(2, strPrompt);
            send(130,status);
            strPrompt = "|en|Time set Successful";
            ACKNOWLEDGE(strPrompt);
        }
        else if(status[STATUS_RESPONSE_CODE] == 2)
        { 
            strPrompt = "|en|Invalid Time entry";
            ACKNOWLEDGE(strPrompt);
        }
        else
        {
            display_response_status(131, status[STATUS_RESPONSE_CODE]);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

VARIABLE Change_RTC_Date
{
    LABEL "Set Date(dd/mm/yyyy)";
    HELP  "|en|Facility to edit date using DD/MM/YYYY format";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ASCII(10);
    DEFAULT_VALUE "10/06/2010";
    
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_change_DATE
{
    LABEL "Set Date(dd/mm/yyyy)";
    HELP  "|en|Facility to edit date using DD/MM/YYYY format";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        add_abort_method(user_abort);
        vassign(Change_RTC_Time, RTC_Time);
        vassign(Change_RTC_Date, RTC_Date);
        iassign(Change_Date_Time_Sel, 0);
        strPrompt = "Set Date(dd/mm/yyyy)";
        GET_DEV_VAR_VALUE(strPrompt, Change_RTC_Date);
        result = send(131,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   
            strPrompt = "|en|Date set in progress";
            DELAY(2, strPrompt);
            send(130,status);
            strPrompt = "|en|Date set Successful";
            ACKNOWLEDGE(strPrompt);
        }
        else if(status[STATUS_RESPONSE_CODE] == 2)
        {
            strPrompt = "|en|Invalid Date entry";
            ACKNOWLEDGE(strPrompt);
        }
        else
        {
            display_response_status(131, status[STATUS_RESPONSE_CODE]);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_WRITE_RTC
{
    NUMBER 131;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            Change_Date_Time_Sel,
            Change_RTC_Time,
            Change_RTC_Date
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid Time entry";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 132 read logged warnings and  faults ----------------*/
VARIABLE faultWarningType
{
    LABEL "faultWarningType";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE INTEGER;
}

VARIABLE log_event_dir
{
    LABEL "log event dir";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE INTEGER;
}

VARIABLE evtIndex
{
    LABEL "Index";
    CLASS DEVICE;
    HANDLING READ;
    #if __TOKVER__ >= 602
    #endif
    TYPE UNSIGNED_INTEGER(4);
    DEFAULT_VALUE 0;
}

VARIABLE evtTime
{
    LABEL "Time";
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII(21);
    DEFAULT_VALUE "17/04/2018 : 13:15:35";
}
VARIABLE event_arg1
{
    LABEL "Userinfo";
    CLASS DEVICE;
    HANDLING READ;
    #if __TOKVER__ >= 602
    #endif
    TYPE UNSIGNED_INTEGER(4);
    DEFAULT_VALUE 0;
}

VARIABLE event_arg2
{
    LABEL "Userextndinfo";
    CLASS DEVICE;
    HANDLING READ;
    #if __TOKVER__ >= 602
    #endif
    TYPE UNSIGNED_INTEGER(4);
    DEFAULT_VALUE 0;
}

VARIABLE evtFilter
{
    LABEL "Filter";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    #endif
    TYPE ENUMERATED
    {
        { 5, "|en|Faults", "Displays History Of Faults" },
        { 4, "|en|Warning", "Displays History Of Warnings" }
    }
    DEFAULT_VALUE 4;
}

VARIABLE evtView
{
    LABEL "Event History";
    HELP "Displays event history from latest to oldest";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    #endif
    TYPE ENUMERATED
    {
        { 0, "|en|Latest Log"}
    }   
    DEFAULT_VALUE 0;
    POST_EDIT_ACTIONS 
    {
        mth_show_events
    }
}

VARIABLE evtType
{
    LABEL "Type";
    CLASS LOCAL;
    HANDLING READ;
    TYPE ENUMERATED(4)
    {
        { 0x2001, "|en|Safety MCU Warning"                                       },
        { 0x2002, "|en|Safety MCU Fault"                                         },
        { 0x2003, "|en|Reserved"                                                 },
        { 0x2004, "|en|Safety MCU Unknown Fault(not defined)"                    },
        { 0x2005, "|en|non safety  MCU Warning"                                  },
        { 0x2006, "|en|non safety  MCU Fault"                                    },
        { 0x2007, "|en|Reserved"                                                 },
        { 0x2008, "|en|non safety MCU Unknown Fault(not defined)"                },
        { 0x2009, "|en|safety cpu assertion failure"                             },
        { 0x200A, "|en|non safety cpu assertion failure"                         },
        { 0x200B, "|en|SDS failure"                                              },
        { 0x200D, "|en|Safety MCU Bootup fail"                                   },
        { 0x200E, "|en|non Safety MCU Bootup fail"                               },

        { 0x80004000, "|en|Alarm 1"                                                  },
        { 0x80004001, "|en|Alarm 2"                                                  },
        { 0x80004002, "|en|Gas Over range"                                           },
        { 0x4003,     "|en|Test Signal "                                             },

        { 0x80005000, "|en|Operating state Fault"                                    },  
        { 0x80005001, "|en|Temperature fault"                                        }, /* HART specific */
        { 0x80005002, "|en|Bad 24V DC Supply"                                        },
        { 0x80005003, "|en|Bad 5V DC Supply"                                         },
        { 0x80005004, "|en|4-20mA Feedback read failure"                             },
        { 0x80005005, "|en|4-20mA 3.3VDC Supply error"                               },
        { 0x80005006, "|en|4-20mA Wiring Fault"                                      },
        { 0x80005007, "|en|4-20mA calibration error"                                 },
        { 0x80005008, "|en|Relay 1 Current monitoring failed"                        },
        { 0x80005009, "|en|Relay 1 Pulse test failed"                                },
        { 0x8000500A, "|en|Relay 1 PSU test failed"                                  },
        { 0x8000500B, "|en|Relay 2 Current monitoring failed"                        },
        { 0x8000500C, "|en|Relay 2 Pulse test failed"                                },
        { 0x8000500D, "|en|Relay 2 PSU test failed"                                  },
        { 0x8000500E, "|en|Relay 3 Current monitoring failed"                        },
        { 0x8000500F, "|en|Relay 3 Pulse test failed"                                },
        { 0x80005010, "|en|Relay 3 PSU test failed"                                  },
        { 0x80005011, "|en|Watchdog diagnostics fail on Temporal Monitoring"         },  /* HART specific */
        { 0x80005012, "|en|AFE diagnostic fault"                                     },
        { 0x80005013, "|en|Diagnostic persistent fault"                              },
        { 0x80005014, "|en|Diagnostic Module fault"                                  },
        { 0x80005015, "|en|Wireless fault"                                           },  /* HART specific */
        { 0x80005016, "|en|Power ON diagnostics fault"                               },
        { 0x80005017, "|en|Power ON diagnostics persistent fault"                    },
        { 0x80005018, "|en|Bad 12v supply(safe)"                                     },
        { 0x80005019, "|en|Bad 3v3 supply(safe)"                                     },
        { 0x8000501A, "|en|Bad 12v supply(non safe)"                                 },
        { 0x8000501B, "|en|Bad 3v3 supply(non safe)"                                 },
        { 0x8000501C, "|en|Bad 5v supply(non safe)"                                  },
        { 0x8000501D, "|en|Bad 1v2 supply(safe)"                                     },
        { 0x8000501E, "|en|Bad 2v5 supply(safe)"                                     },
        { 0x8000501F, "|en|Bad 2v5 ref supply"                                       },
        { 0x80005020, "|en|Bad 8v ref supply"                                        },
        { 0x80005021, "|en|Bad 5v heater supply"                                     },
        { 0x80005022, "|en|Bad 1v2 supply(non safe)"                                 },
        { 0x80005023, "|en|Bad mA external power supply"                             },
        { 0x80005024, "|en|mA read back error"                                       },
        { 0x5025,     "|en|Safe MCU core diagnostics fault"                          },
        { 0x5026,     "|en|Safe MCU application diagnostics fault"                   },
        { 0x5027,     "|en|Safe MCU core diagnostics ESM error"                      },
        { 0x80005028, "|en|Afe High gain fault"                                      },
        { 0x80005029, "|en|Afe medium gain fault"                                    },
        { 0x8000502A, "|en|Afe Low pass filter fault"                                },
        { 0x8000502B, "|en|AFE ADC fault"                                            },
        { 0x8000502C, "|en|mA loop diagnostic fault"                                 },
        { 0x502D,     "|en|ADC diagnostics fault"                                    },
        { 0x502E,     "|en|MIBSPI3 diagnostics fault"                                },
        { 0x502F,     "|en|mA ADC MIBSPI5 diagnostics fault"                         },
        { 0x5030,     "|en|mA ADC MIBSPI5 diagnostics fault"                         },
        { 0x5031,     "|en|ADC1 self test fail fault"                                },
        { 0x5032,     "|en|ADC2 self test fail fault"                                },
        { 0x5033,     "|en|ADC calibration fault"                                    },
        { 0x80005034, "|en|4-20 mA loop drift fault"                                 },

        { 0x80006000, "|en|Operating state Warning "                                 },
        { 0x80006001, "|en|Wireless Diagnostic Fail  "                               },
        { 0x80006002, "|en|Diagnostic Module Warning "                               },
        { 0x80006003, "|en|Maintenance flag enabled "                                },
        { 0x80006004, "|en|Power ON diagnostics warning "                            },
        { 0x80006005, "|en|Bad 24v DC Supply warning"                                },
        { 0x80006006, "|en|Bad 5v Supply warning (safe)"                             },
        { 0x80006007, "|en|Bad 12v Supply warning (safe)"                            },
        { 0x80006008, "|en|Bad 3v3 Supply warning (safe)"                            },
        { 0x80006009, "|en|Bad 12v Supply warning (non safe)"                        },
        { 0x8000600A, "|en|Bad 3v3 Supply warning (non safe)"                        },
        { 0x8000600B, "|en|Bad 5v Supply warning (non safe)"                         },
        { 0x8000600C, "|en|Bad 1v2 Supply warning (safe)"                            },
        { 0x8000600D, "|en|Bad 2v5 Supply warning (safe)"                            },
        { 0x8000600E, "|en|Bad 2v5 ref Supply warning"                               },
        { 0x8000600F, "|en|Bad 8v ref Supply warning"                                },
        { 0x80006010, "|en|Bad 5v heater Supply warning"                             },
        { 0x80006011, "|en|Bad 1v2 Supply warning(non safe)"                         },
        { 0x80006012, "|en|Date and Time not set warning"                            },
        { 0x80006013, "|en|Non Safe MCU diagnostic fail warning"                     },
        { 0x6014,     "|en|RTC SPI4 fail warning"                                    },
        { 0x6015,     "|en|IPC MIBSPI1 fail warning"                                 },

        { 0x80007000, "|en|4-20mA loop calibration started "                     },
        { 0x7000, "|en|4-20mA loop calibration completed "                       },
        { 0x7001, "|en|Date and Time changed  "                                  },
        { 0x7002, "|en|Operating state change "                                  },
        { 0x7003, "|en|Reserved "                                                },
        { 0x80007004, "|en|Wireless module active "                              },
        { 0x7004, "|en|Wireless Module deactive"                                 },
        { 0x7005, "|en|Reset request"                                            },
        { 0x7006, "|en|Configuration backup "                                    },
        { 0x7007, "|en|Configuration restored "                                  },
        { 0x7008, "|en|Session Error "                                           },
        { 0x7009, "|en|PIN Verified "                                            },
        { 0x700A, "|en|PIN Changed "                                             },
        { 0x700B, "|en|Power on start"                                           },
        { 0x700C, "|en|Invalid RS 485 configuration"                             },
        { 0x700D, "|en|Power On NVM fail"                                        },
        { 0x700E, "|en|Function test overdue"                                    },

        { 0x8000, "|en|Session open"                                             },
        { 0x8001, "|en|Session close"                                            },
        { 0x8002, "|en|Detector configuration modified "                         },
        { 0x8003, "|en|Clear Latch request event "                               },
        { 0x80008004, "|en|Relay 1 latched "                                     },
        { 0x8004, "|en|Relay 1 latch cleared "                                   },
        { 0x80008005, "|en|Relay 2 latched "                                     },
        { 0x8005, "|en|Relay 2 latch cleared "                                   },
        { 0x80008006, "|en|Relay 3 latched "                                     },
        { 0x8006, "|en|Relay 3 latch cleared "                                   },
        { 0x80008007, "|en|4-20mA Loop Force Active"                             },
        { 0x8007, "|en|4-20mA Loop Force cleared"                                },
        { 0x80008008, "|en|Relay 1 Force Active"                                 },
        { 0x8008, "|en|Relay 1 Force cleared"                                    },
        { 0x80008009, "|en|Inhibit Active"                                       },
        { 0x8009, "|en|Inhibit cleared"                                          },
        { 0x800A, "|en|Firmware Updated"                                         },
        { 0x800B, "|en|Detector restarted "                                      },
        { 0x8000800C, "|en|Relay 2 Force Active"                                 },
        { 0x800C, "|en|Relay 2 Force cleared"                                    },
        { 0x8000800D, "|en|Relay 3 Force Active"                                 },
        { 0x800D, "|en|Relay 3 Force cleared"                                    },
        { 0x8000800E, "|en|LED Force Active"                                     },
        { 0x800E, "|en|LED Force cleared"                                        },
        { 0x8000800F, "|en|4-20mA Force timeout "                                },
        { 0x80008010, "|en|Relay 1 Force auto timeout "                          },
        { 0x80008020, "|en|Relay 2 Force auto timeout "                          },
        { 0x80008030, "|en|Relay 3 Force auto timeout "                          },
        { 0x80008040, "|en|LED Force auto timeout "                              },
        { 0x80008041, "|en|Inhibit auto timeout "                                },
       
        {  0x4000,"|en|Alarm 1 cleared"                                                  },
        {  0x4001,"|en|Alarm 2 cleared"                                                  },
        {  0x4002,"|en|Gas Over range cleared"                                           },

        {  0x5000,"|en|Operating state Fault cleared"                                    },  
        {  0x5002,"|en|Bad 24V DC Supply cleared"                                        },
        {  0x5003,"|en|Bad 5V DC Supply cleared"                                         },
        {  0x5004,"|en|4-20mA Feedback read failure cleared"                             },
        {  0x5005,"|en|4-20mA 3.3VDC Supply error cleared"                               },
        {  0x5006,"|en|4-20mA Wiring Fault cleared"                                      },
        {  0x5007,"|en|4-20mA calibration error cleared"                                 },
        {  0x5008,"|en|Relay 1 Current monitoring fail cleared"                          },
        {  0x5009,"|en|Relay 1 Pulse test fail cleared"                                  },
        {  0x500A,"|en|Relay 1 PSU test fail cleared"                                    },
        {  0x500B,"|en|Relay 2 Current monitoring fail cleared"                          },
        {  0x500C,"|en|Relay 2 Pulse test fail cleared"                                  },
        {  0x500D,"|en|Relay 2 PSU test fail cleared"                                    },
        {  0x500E,"|en|Relay 3 Current monitoring fail cleared"                          },
        {  0x500F,"|en|Relay 3 Pulse test fail cleared"                                  },
        {  0x5010,"|en|Relay 3 PSU test fail cleared"                                    },
        {  0x5012,"|en|AFE diagnostic fault cleared"                                     },
        {  0x5013,"|en|Diagnostic non recover fault cleared"                             },
        {  0x5014,"|en|Diagnostic Module fault cleared"                                  },
        {  0x5016,"|en|Power ON diagnostics recovery fault cleared"                      },
        {  0x5017,"|en|Power ON diagnostics Non recovery fault cleared"                  },
        {  0x5018,"|en|Bad 12v supply(safe) cleared"                                     },
        {  0x5019,"|en|Bad 3v3 supply(safe) cleared"                                     },
        {  0x501A,"|en|Bad 12v supply(non safe) cleared"                                 },
        {  0x501B,"|en|Bad 3v3 supply(non safe) cleared"                                 },
        {  0x501C,"|en|Bad 5v supply(non safe) cleared"                                  },
        {  0x501D,"|en|Bad 1v2 supply(safe) cleared"                                     },
        {  0x501E,"|en|Bad 2v5 supply(safe) cleared"                                     },
        {  0x501F,"|en|Bad 2v5 ref supply cleared"                                       },
        {  0x5020,"|en|Bad 8v ref supply cleared"                                        },
        {  0x5021,"|en|Bad 5v heater supply cleared"                                     },
        {  0x5022,"|en|Bad 1v2 supply(non safe) cleared"                                 },
        {  0x5023,"|en|Bad mA external power supply cleared"                             },
        {  0x5024,"|en|mA read back error cleared"                                       },
        {  0x5028,"|en|Afe High gain fault cleared"                                      },
        {  0x5029,"|en|Afe medium gain fault cleared"                                    },
        {  0x502A,"|en|Afe Low pass filter fault cleared"                                },
        {  0x502B,"|en|AFE ADC fault cleared"                                            },
        {  0x502C,"|en|mA loop diagnostic fault cleared"                                 },
        {  0x5034,"|en|4-20 mA loop drift fault cleared"                                 },
        
        {  0x6000,"|en|Operating state Warning cleared"                                  },
        {  0x6001,"|en|Wireless Diagnostic Fail cleared"                                 },
        {  0x6002,"|en|Diagnostic Module Warning cleared"                                },
        {  0x6003,"|en|Maintenance Flag Warning cleared"                                 },
        {  0x6004,"|en|Power ON diagnostics warning cleared"                             },
        {  0x6005,"|en|Bad 24v DC Supply warning cleared"                                },
        {  0x6006,"|en|Bad 5v Supply warning (safe) cleared"                             },
        {  0x6007,"|en|Bad 12v Supply warning (safe) cleared"                            },
        {  0x6008,"|en|Bad 3v3 Supply warning (safe) cleared"                            },
        {  0x6009,"|en|Bad 12v Supply warning (non safe) cleared"                        },
        {  0x600A,"|en|Bad 3v3 Supply warning (non safe) cleared"                        },
        {  0x600B,"|en|Bad 5v Supply warning (non safe) cleared"                         },
        {  0x600C,"|en|Bad 1v2 Supply warning (safe) cleared"                            },
        {  0x600D,"|en|Bad 2v5 Supply warning (safe) cleared"                            },
        {  0x600E,"|en|Bad 2v5 ref Supply warning cleared"                               },
        {  0x600F,"|en|Bad 8v ref Supply warning cleared"                                },
        {  0x6010,"|en|Bad 5v heater Supply warning cleared"                             },
        {  0x6011,"|en|Bad 1v2 Supply warning(non safe) cleared"                         },
        {  0x6012,"|en|Date and Time not set warning cleared"                            },
        {  0x6013,"|en|Non Safe MCU diagnostic fail warning cleared"                     },

        { 0xFFFFFFFF, "|en|No more events"                                               }
    }
    DEFAULT_VALUE 0xFFFFFFFF;
}

/* Method to show event information (type of event and time stamp) */
METHOD mth_show_events
{   
    LABEL "Event History";
    HELP "Displays the event type and the time it was reported";
    DEFINITION
    {
        DD_STRING strPrompt;
        DD_STRING strPrompt1;
        DD_STRING strPrompt2;
        char status[STATUS_SIZE];
        int res;
        long indx;
        int evtTypeLog;
        int error_cnt;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        
        faultWarningType = SELECT_FROM_LIST("Log Type","Internal and Security Events;Alarms/Proof Test;Service ");
        error_cnt = 0;
        /* Request for event history log */
        /* To get latest event log */
        iassign(log_event_dir, 0); 
        do
        {
            /* get index in evtIndex */
            send(132,status);
            if(status[STATUS_RESPONSE_CODE] == 0)
            {
                switch(evtIndex)
                {
                    case 0x4003:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "%{event_arg1}\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x5001:
                    {
                        iassign(evtIndex, 0x80005001);
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "%{event_arg1}\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x5011:
                    {
                        iassign(evtIndex, 0x80005011);
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "%{event_arg1}\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x5015:
                    {
                        iassign(evtIndex, 0x80005015);
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x5025:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "%{event_arg1}\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x5026:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "%{event_arg1}\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x5027:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "%{event_arg1}\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x502D:
                    case 0x502E:
                    case 0x502F:
                    case 0x5030:
                    case 0x5031:
                    case 0x5032:
                    case 0x5033:
                    case 0x5034:
                    case 0x6014:
                    case 0x6015:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + " - "+"%{event_arg1}\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x7002:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        indx = ivar_value(event_arg1);
                        res = get_enum_string(Current_OpMode, indx, strPrompt2);
                        if(res == BI_ERROR)
                        {
                            strPrompt2 = "Invalid state";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 +" - "+ strPrompt2+"\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    case 0x80008009:
                    case 0x8009:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        indx = ivar_value(event_arg1);
                        if(indx == 1)
                        {
                            strPrompt2 = "By Mobile App";
                        }
                        else if(indx == 2)
                        {
                            strPrompt2 = "By HART";
                        }
                        else
                        {
                            strPrompt2 = " ";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 +" - "+ strPrompt2+"\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                    break;
                    default:
                    {
                        indx = ivar_value(evtIndex);
                        res = get_enum_string(evtType, indx, strPrompt1);
                        if(res == BI_ERROR)
                        {
                            strPrompt1 = "Invalid Index";
                        }
                        strPrompt = "Event Information \n"
                                    "Type  : " + strPrompt1 + "\n" +
                                    "Time  :%{evtTime}     ";
                        ACKNOWLEDGE(strPrompt);
                    }
                }
                error_cnt = 0;
            }
            else if(status[STATUS_RESPONSE_CODE] == 32)
            {
                display_response_status(132, status[STATUS_RESPONSE_CODE]);
                error_cnt++;
            }
            /* To get older event log */
            iassign(log_event_dir, 2); 
        }while( (indx != 0xFFFFFFFF) && (error_cnt < 2) );
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_READ_EVENT_HISTORY
{
    NUMBER 132;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            faultWarningType,
            log_event_dir,
            login_level
        }
        REPLY
        {
            response_code, device_status,
            evtIndex,
            evtTime,
            event_arg1,
            event_arg2
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 134 Read warnings and faults ----------------*/
VARIABLE evtView_warnings
{
    LABEL "Active Warnings";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    #endif
    TYPE ENUMERATED
    {
        { 0, "|en|First" },
        { 2, "|en|Next"}
    }   
    POST_EDIT_ACTIONS 
    {
        mth_show_warnings
    }
    DEFAULT_VALUE 0;
}

VARIABLE eventDir
{
    LABEL "eventDir";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    #if __TOKVER__ >= 602
    #endif
    TYPE INTEGER;
    DEFAULT_VALUE 0;
}

VARIABLE eventFilter
{
    LABEL "eventFilter";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE INTEGER;
    DEFAULT_VALUE 2;
}

METHOD mth_show_warnings
{   
    LABEL "Active Warnings";
    HELP "Displays active warnings";
    DEFINITION
    {
        DD_STRING strPrompt;
        DD_STRING strPrompt1;
        char status[STATUS_SIZE];
        int res;
        long indx;


        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        /* Request for Active Warning */
        iassign(faultWarningType, 0);
        /* To get first event */
        iassign(eventDir, 0);
        /* If the user's present logged-in level is greater than Level1 */
        if(login_level > LEVEL1 ) 
        {
            /* Get all warnings */
            iassign(eventFilter, 1);      
        }
        else
        {
            /* Get only major warnings */
            iassign(eventFilter, 2);
        }
        do{
        
            /* get index value in evtIndex */
            send(134,status);
            indx = ivar_value(evtIndex);
            res = get_enum_string(evtType, indx, strPrompt1);
            if(res == BI_ERROR)
            {
                strPrompt1 = "Invalid Index";
            }
            if(indx == 0xFFFFFFFF)
            {
                strPrompt1 = "No more warnings";
                strPrompt = "Event Information   \n"
                            "Type  : " + strPrompt1;
            }
            else
            {
                strPrompt = "Event Information   \n"
                            "Type  : " + strPrompt1;
            }
            ACKNOWLEDGE(strPrompt);
            /* To get next available event */
            iassign(eventDir, 2); 
        }while(indx != 0xFFFFFFFF);
    }
}

VARIABLE evtView_faults
{
    LABEL "Active Faults";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    #if __TOKVER__ >= 602
    #endif
    TYPE ENUMERATED
    {
        { 0, "|en|First" },
        { 2, "|en|Next"}
    }   
    POST_EDIT_ACTIONS 
    {
        mth_show_faults
    }
    DEFAULT_VALUE 0;
}

METHOD mth_show_faults
{   
    LABEL "Active Faults";
    HELP "Displays active faults";
    DEFINITION
    {
        DD_STRING strPrompt;
        DD_STRING strPrompt1;
        char status[STATUS_SIZE];
        int res;
        long indx;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        /* Request for Active Fault */
        iassign(faultWarningType, 1);
        /* To get first event */
        iassign(eventDir, 0); 
        /* if user logged in level greater than level1 */
        if(login_level > LEVEL1 ) 
        {   
            /* Get all faults */
            iassign(eventFilter, 1);      
        }
        else
        {
            /* Get only major faults */
            iassign(eventFilter, 2);
        }
        do
        {
            /* get index value in evtIndex */
            send(134,status); 
            indx = ivar_value(evtIndex);
            res = get_enum_string(evtType,indx,strPrompt1);
            if(res == BI_ERROR)
            {
                strPrompt1 = "Invalid Index";
            }
            if(indx == 0xFFFFFFFF)
            {
                strPrompt1 = "No more faults";
                strPrompt = "Event Information   \n"
                            "Type  : " + strPrompt1;
            }
            else
            {
                strPrompt = "Event Information   \n"
                            "Type  : " + strPrompt1;
            }
            ACKNOWLEDGE(strPrompt);
            /* To get next available event */
            iassign(eventDir, 2); 
        }while (indx != 0xFFFFFFFF);
    }
}

COMMAND READ_DIAGNOSTICS
{
    NUMBER 134;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            faultWarningType, 
            eventDir,
            eventFilter
        }
        REPLY
        {
            response_code, device_status, 
            evtIndex
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 133 Field data -----------------*/
VARIABLE fTemperature
{
    LABEL "Field Temperature";
    HELP  "Analog Value - Represents Field Temperature";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    CONSTANT_UNIT "degC";
}

VARIABLE fTargetCurrent
{
    LABEL "Target Current";
    HELP  "Analog Value - Represents mA current to be driven by 4-20mA loop";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    CONSTANT_UNIT "mA";
}

VARIABLE fMeasuredCurrent
{
    LABEL "Measured Current";
    HELP  "Analog value - Represents mA measured current driven by 4-20mA loop";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    CONSTANT_UNIT "mA";
}

VARIABLE f5vSupply
{
    LABEL "Analog 5v Rail";
    HELP  "Analog value - Represents 5v power rail";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    CONSTANT_UNIT "v ";
}

VARIABLE fSupplyVoltage
{
    LABEL "Analog 24v Rail";
    HELP  "Analog value - Represents 24v power rail";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    CONSTANT_UNIT "v ";
}

VARIABLE ifunctiontestdue
{
    LABEL "Function Test Due";
    HELP  "Represents due time for next Function Test";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE INTEGER(2);
    CONSTANT_UNIT "days";
}

VARIABLE relay1state
{
    LABEL "Relay1 State";
    HELP  "Represents operating state of relay1";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED
    {
        {0 , "OPEN"}, 
        {1 , "CLOSE"}
    }
}

VARIABLE relay2state
{
    LABEL "Relay2 State";
    HELP  "Represents operating state of relay2";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED
    {
        {0 , "OPEN"}, 
        {1 , "CLOSE"}
    }
}

VARIABLE relay3state
{
    LABEL "Relay3 State";
    HELP  "Represents operating state of relay3";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED
    {
        {0 , "OPEN"}, 
        {1 , "CLOSE"}
    }
}

COMMAND CMD_READ_FIELD_DATA 
{
    NUMBER 133;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            fTemperature,
            fTargetCurrent,
            fMeasuredCurrent,
            f5vSupply,
            fSupplyVoltage,
            ifunctiontestdue,
            relay1state,
            relay2state,
            relay3state
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        8,  DATA_ENTRY_ERROR, [invalid_selection];
        16, MODE_ERROR,       [access_restricted];
    }
}


/*------------------- COMMAND 135 Simulate alarm, warning and fault -----------------*/
VARIABLE sim_alm_flt
{
    LABEL "|en|Simulate";
    HELP "|en|Facility to simulate alarm, warning or fault";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Alarm"},
        {1,"|en|Warning"},
        {2,"|en|Fault"},
        {3,"|en|End"}
    }
    POST_EDIT_ACTIONS
    {
        mth_sim_alm_flt
    }
}

METHOD mth_sim_alm_flt
{
    LABEL "Simulate Alarm Fault";
    HELP  "|en|Facility to simulate alarm, warning or fault";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int use_abort_method;
        int choice;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        add_abort_method(user_return_to_normal);

        use_abort_method = 0;
        do
        {
           iassign(sim_alm_flt, SELECT_FROM_LIST("Simulate Alarm or Warning or Fault","Alarm;Warning;Fault;End Simulation"));
           
           send(135,status);
           use_abort_method = 1;
           display_response_status(135, status[STATUS_RESPONSE_CODE]);
           if(use_abort_method)
           {
               add_abort_method(simulate_normal);
           }
        }while (sim_alm_flt < 3);
        user_return_to_normal();
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD simulate_normal
{
    LABEL "simulate_normal";
    HELP  "";
    DEFINITION 
    {
       char status[STATUS_SIZE];

       iassign(sim_alm_flt, 3);
       send(135,status);
    }
}

COMMAND CMD_SIMULATE_ALARMS_FAULT
{
    NUMBER 135;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            sim_alm_flt
        }
        REPLY
        {
            response_code, device_status     
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       "|en|Device in multidrop mode";
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 136 & 137 Read and configure Inhibit timeout -----------------*/
VARIABLE Inhibit_Timeout
{
    LABEL "Inhibit Timeout";
    HELP  "|en|Inhibit Timeout\n\n '0' means no timeout \n\n Range: 60 - 600Sec";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER(2)
    {
        MIN_VALUE1 0;
        MAX_VALUE1 0;
        MIN_VALUE2 60;
        MAX_VALUE2 600;
    }
    DEFAULT_VALUE 60;
    CONSTANT_UNIT "|en|Sec ";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_READ_INHIBIT_TIMEOUT
{
    NUMBER 136;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, Inhibit_Timeout
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_WRITE_INHIBIT_TIMEOUT
{
    NUMBER 137;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            Inhibit_Timeout
        }
        REPLY
        {
            response_code, device_status, Inhibit_Timeout
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 138 Configure the alarm threshold -----------------*/
VARIABLE alarm_set_point
{
    LABEL "Alarm Set Point";
    HELP  "Analog value - Represents Alarm Set Point in dB.\nRange 40 - 120dB";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE 40.0;
        MAX_VALUE 120.0;
        DISPLAY_FORMAT ".1f";
        EDIT_FORMAT ".1f";
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 40.0;
    CONSTANT_UNIT "|en|dB";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

VARIABLE alarm1_mA
{
    LABEL "Alarm 1 Level";
    HELP  "Analog value - Represents Alarm 1 level in mA.\nRange 4 - 20mA";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE 4.0;
        MAX_VALUE 20.0;
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 16.0;
    CONSTANT_UNIT "|en|mA";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

VARIABLE alarm2_mA
{
    LABEL "Alarm 2 Level";
    HELP  "Analog value - Represents Alarm 2 level in mA.\nRange 4 - 20mA";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE 4.0;
        MAX_VALUE 20.0;
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 18.0;
    CONSTANT_UNIT "|en|mA";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

VARIABLE alarm_delay
{
    LABEL "Alarm Delay";
    HELP  "Represents Alarm Delay in Sec.\nRange 1 - 600Sec";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER(2)
    {
        MIN_VALUE 1;
        MAX_VALUE 600;
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 60;
    CONSTANT_UNIT "|en|Sec";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_SET_ALARM_CONFIG
{
    NUMBER 138;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            alarm_set_point,alarm1_mA,alarm2_mA,alarm_delay
        }
        REPLY
        {
            response_code, device_status,
            alarm_set_point,alarm1_mA,alarm2_mA,alarm_delay
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 139 Read alarm threshold -----------------*/
COMMAND CMD_READ_ALARM_CONFIG
{
    NUMBER 139;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            alarm_set_point,alarm1_mA,alarm2_mA,alarm_delay
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 140 Authenticate the login level -----------------*/
VARIABLE login_level
{
    LABEL "User logged in as:";
    HELP  "|en|Represents Logged in user level";
    CLASS LOCAL;
    HANDLING READ & WRITE;
    TYPE ENUMERATED
    {
        { 0, "Guest", "No Access to Configurations"           },
        { 1, "Engineer", "Maintenance Engineer"               },
        { 2, "Honeywell Engineer", "Honeywell Field Engineer" }
    }
    DEFAULT_VALUE 0;
}

VARIABLE password
{
    LABEL "Password";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE PASSWORD(8);
    DEFAULT_VALUE "00000000";
}

VARIABLE login_Status
{
    LABEL "Login Status";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE INTEGER;
}

VARIABLE Dev_login_level
{
    LABEL "Change Login";
    HELP  "Switch login level";
    CLASS DEVICE;
    HANDLING WRITE;    
    TYPE ENUMERATED
    {
        { 0, "Guest", "No Access to Configuration" },
        { 1, "Engineer", "Maintainance Engineer" },
        { 2, "Honeywell Engineer", "Honeywell Field Engineer" }   
    }
    DEFAULT_VALUE 0;
}

METHOD mth_get_pw
{
    LABEL "Change login profile";
    HELP  "Switch login level\nGuest - No Access to Configuration\nEngineer - Maintainance Engineer\nHoneywell Engineer - Honeywell Field Engineer";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        iassign(Dev_login_level, SELECT_FROM_LIST("Change Login","Guest;Engineer;Honeywell Engineer"));
        if(Dev_login_level > LEVEL0)
        {
            GET_DEV_VAR_VALUE("|en|Enter Password",password);
        }
        iassign(login_level, Dev_login_level);
        result = send(140,status);   
        if(!result)
        {
            if(ivar_value(login_Status) == 0) 
            {   
                DISPLAY("|en|Login Successful");
            }
            else
            {
                DISPLAY("|en|Invalid Password");
            }
        }
        else
        {
            display_response_status(140, status[STATUS_RESPONSE_CODE]);
        }
    }
}

REFRESH login
{
    login_level : Dev_login_level
}

COMMAND  CMD_USER_LOG_IN
{
    NUMBER 140;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            login_level,
            password
        }
        REPLY
        {
            response_code, device_status,
            login_level,
            login_Status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          "|en|Login Successful!";
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       "|en|Invalid Password";
        32, MODE_ERROR,       "|en|Invalid Login Requested";
    }
}

/*------------------- COMMAND 142 Change password for current login level -----------------*/
VARIABLE Change_Password
{
    LABEL "Change Password";
    HELP  "|en|Change the password for the current level";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE PASSWORD(8);
    
    VALIDITY IF((login_level == LEVEL0) || (login_level == LEVEL1) ){TRUE;}ELSE {FALSE;}
}

METHOD mth_set_pw
{
    LABEL "Change Password";
    HELP  "|en|Change the password for the current level";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        strPrompt = "Change Password";
        GET_DEV_VAR_VALUE(strPrompt, Change_Password);
        result = send(142,status);
        if(!result)
        {
            strPrompt = "Password Changed!";
        }
        else
        {
            strPrompt = "Password Change Not Successful!";
        }
        ACKNOWLEDGE(strPrompt);
    }
    VALIDITY IF((login_level == LEVEL0) || (login_level == LEVEL1) ){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_USER_PWD_CHANGE
{
    NUMBER 142;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            login_level,
            Change_Password
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       "|en|Invalid Password";
        32, MODE_ERROR,       "|en|Invalid Login Level";
    }
}

/*------------------- COMMAND 143 & 144 Read & Configure Relays ----------------*/
VARIABLE Relay1
{
    LABEL "|en|Fault (R1)";
    HELP "|en|Represents relay1 default state";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Off"},
        {1,"|en|On"}
    }
}

VARIABLE Relay1_latch
{
    LABEL "|en|Fault (R1) Latch";
    HELP "|en|Represents fault relay latch status";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Disable"},
        {1,"|en|Enable"}
    }
}

VARIABLE Relay2
{
    LABEL "|en|Alam 1(R2)";
    HELP "|en|Represents alarm 1 relay default state";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Off"},
        {1,"|en|On"}
    }
}

VARIABLE Relay2_latch
{
    LABEL "|en|Alam 1(R2) Latch";
    HELP "|en|Represents alam 1 relay latch status";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Disable"},
        {1,"|en|Enable"}
    }
}

VARIABLE Relay3
{
    LABEL "|en|Alam 2(R3)";
    HELP "|en|Represents alarm 2 relay default state";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Off"},
        {1,"|en|On"}
    }
}

VARIABLE Relay3_latch
{
    LABEL "|en|Alam 2(R3) Latch";
    HELP "|en|Represents alam 2 relay latch status";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Disable"},
        {1,"|en|Enable"}
    }
}

VARIABLE relay_indx
{
    LABEL "|en|Relay Index";
    HELP "|en|Relay selection Fault(R1) or Alam 1(R2) or Alam 2(R3)";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE UNSIGNED_INTEGER(1);
}

METHOD mth_Relay1
{
    LABEL "Fault (R1)";
    HELP  "|en|Configure relay1 default state to Off or On";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
            Relay1 = SELECT_FROM_LIST("Select relay1 default state to","Off;On") ;
			
            result = send(144,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure relay1 default state Successful";
            }
            else
            {
                strPrompt = "|en|Configure relay1 default NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_Relay1_latch
{
    LABEL "Fault (R1) Latch";
    HELP  "|en|Configure relay1 latch default state to Disable or Enable";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
            Relay1_latch = SELECT_FROM_LIST("Select relay1 default state to","Disable;Enable") ;
			
            result = send(144,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure relay1 latch default state Successful";
            }
            else
            {
                strPrompt = "|en|Configure relay1 latch default state NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_Relay2
{
    LABEL "Alam 1(R2)";
    HELP  "|en|Configure relay2 default state to Off or On";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
            Relay2 = SELECT_FROM_LIST("Select relay2 default state to","Off;On") ;
			
            result = send(144,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure relay2 default state Successful";
            }
            else
            {
                strPrompt = "|en|Configure relay2 default state NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_Relay2_latch
{
    LABEL "Alam 1(R2) Latch";
    HELP  "|en|Configure relay2 latch default state to Disable or Enable";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
            Relay2_latch = SELECT_FROM_LIST("Select relay2 default state to","Disable;Enable") ;
			
            result = send(144,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure relay2 latch default state Successful";
            }
            else
            {
                strPrompt = "|en|Configure relay2 latch default state NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_Relay3
{
    LABEL "Alam 2(R3)";
    HELP  "|en|Configure relay3 default state to Off or On";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
            Relay3 = SELECT_FROM_LIST("Select relay3 default state to","Off;On") ;
			
            result = send(144,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure relay3 default state Successful";
            }
            else
            {
                strPrompt = "|en|Configure relay3 default state NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_Relay3_latch
{
    LABEL "Alam 2(R3) Latch";
    HELP  "|en|Configure relay3 latch default state to Disable or Enable";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
            Relay3_latch = SELECT_FROM_LIST("Select relay3 default state to","Disable;Enable") ;
			
            result = send(144,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure relay3 latch default state Successful";
            }
            else
            {
                strPrompt = "|en|Configure relay3 latch default state NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_READ_RELAY_DFLT_CONFIG
{
    NUMBER 143;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, Relay1, Relay1_latch, Relay2, Relay2_latch, Relay3, Relay3_latch
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_WRITE_RELAY_DFLT_CONFIG
{
    NUMBER 144;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            Relay1, Relay1_latch, Relay2, Relay2_latch, Relay3, Relay3_latch
        }
        REPLY
        {
            response_code, device_status, Relay1, Relay1_latch, Relay2, Relay2_latch, Relay3, Relay3_latch
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 146 & 147 Read and configure Force timeout -----------------*/
VARIABLE Force_Timeout
{
    LABEL "Force Timeout";
    HELP  "|en|Facility to edit force timeout in seconds\n\n '0' means no time out\n\n Range: 0 - 600Sec";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER(2)
    {
        MIN_VALUE 0;
        MAX_VALUE 600;
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 0;
    CONSTANT_UNIT "|en|Sec ";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_READ_FORCE_TIMEOUT
{
    NUMBER 146;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, Force_Timeout
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_WRITE_FORCE_TIMEOUT
{
    NUMBER 147;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            Force_Timeout
        }
        REPLY
        {
            response_code, device_status, Force_Timeout
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 151 & 152 Read and write Function Test Inerval----------------*/
VARIABLE Function_Test_Interval
{
    LABEL "Function Test Interval";
    HELP  "|en|Represents function test interval.\nRange allowed 0 - 1825 days";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER(2)
    {
        MIN_VALUE 0;
        MAX_VALUE 1825;
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 0;
    CONSTANT_UNIT "|en|Days";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_READ_FUNCTION_TEST_INTVL
{
    NUMBER 151;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, Function_Test_Interval
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_WRITE_FUNCTION_TEST
{
    NUMBER 152;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            Function_Test_Interval
        }
        REPLY
        {
            response_code, device_status, Function_Test_Interval
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 153 & 154 Read and write Algorithm Mode----------------*/
VARIABLE Algorithm_Mode
{
    LABEL "Algorithm Mode";
    HELP  "|en|Algorithm Mode which can contain any one of the following status\n\n"
                          "Standard\n"
                          "Focused\n";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Standard"},
        {1,"|en|Focused"}
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 0;
}

VARIABLE Algorithm_Mode_Rd
{
    LABEL "Algorithm Mode";
    HELP  "|en|Algorithm Mode which can contain any one of the following status\n\n"
                          "Standard\n"
                          "Focused\n";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Standard"},
        {1,"|en|Focused"}
    }
    DEFAULT_VALUE 0;
}

METHOD mth_inhibitinfo
{
    LABEL "Inhibit Info";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        send(129,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else
        {
            send(180,status);
            if( Inhibit_Source_Rd == 1 )
            {
                ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
                process_abort();
            }
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_READ_ALGO_MODE
{
    NUMBER 153;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, Algorithm_Mode_Rd
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_WRITE_ALGO_MODE
{
    NUMBER 154;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            Algorithm_Mode
        }
        REPLY
        {
            response_code, device_status, Algorithm_Mode
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 156  Force Relay ----------------*/
VARIABLE relay_sel
{
    LABEL "|en|Force Relay";
    HELP "|en|Facility to force Fault(R1) or Alarm 1(R2) or Alarm 2(R3)";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Fault (R1)"},
        {1,"|en|Alarm 1(R2)"},
        {2,"|en|Alarm 2(R3)"},
        {3,"|en|End"}
    }
}

VARIABLE relay_state
{
    LABEL "|en|Relay State";
    HELP "|en|Facility to force relay Energized or de Energized";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ENUMERATED(1)
    { 
        {0,"|en|Energized"},
        {1,"|en|de Energized"}
    }
}

METHOD mth_relay_test
{
    LABEL "Set Force Relay";
    HELP  "|en|Facility to force relay state";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char strPrompt[60];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        while(1)
        {
        
        iassign(relay_sel, SELECT_FROM_LIST("Select the Relay to force","Fault (R1);Alarm 1(R2);Alarm 2(R3)"));
        iassign(relay_state, SELECT_FROM_LIST("Select the state to force","NORMAL;OFF;ON;"));
        send(156,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   
            if(relay_state == 0)
            {
                strPrompt = "|en|Relay to NORMAL Successful";
            }
            else
            {
                strPrompt = "|en|Force Relay Successful";
            }
        }
        else
        {
            if(relay_state == 0)
            {
                strPrompt = "|en|Relay to NORMAL NOT Successful";
            }
            else
            {
                strPrompt = "|en|Force Relay NOT Successful";
            }
        }
        ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_FORCE_RELAY
{
    NUMBER 156;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            relay_sel, relay_state
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 157  Force LED state ----------------*/
VARIABLE led_state
{
    LABEL "|en|Force LED State";
    HELP "|en|Facility to force LED to different states";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE UNSIGNED_INTEGER(1);
    //{ 
        /*  LED force setting
        Bit 0 - Alarm
        Bit 1 - Over Range
        Bit 2 - Fault
        Bit 3 - Inhibit
        Bit 4 - Warning
        Bit 5 - Reserved
        Bit 6 - Reserved
        Bit 7 - Reserved
        Bit 8 - BT link status (disconnected / connected)
        Bit 9 - BT session status (handshake / established)
        Bit 10 - Dummy/normal
        Bit 11 to 15 reserved
        
        {0,"|en|Alarm"},
        {1,"|en|Over Range"},
        {2,"|en|Fault"},
        {3,"|en|Inhibit"},
        {4,"|en|Warning"},
        {8,"|en|BT link status"},
        {9,"|en|BT session status"}*/
    //}
}

METHOD mth_led_test
{
    LABEL "Force LEDs";
    HELP  "|en|Facility to force led state";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char strPrompt[60];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        led_state = SELECT_FROM_LIST("Select the LED state to force",         "Normal;"
                                                                              "Alarm/Over Range;"                                                                                                                                                     
                                                                              "Fault;"                                                                           
                                                                              "Inhibit;"                                                                           
                                                                              "Warning;"                                                                           
                                                                              "BT link status;"                                                                           
                                                                              "BT session status;"
                                                                              "End;");

        if(led_state == 0)
        {
            iassign(led_state, 11); 
        }
        else if(led_state == 1)
        {
            iassign(led_state, 1); 
        }
        else if(led_state == 2) 
        {
            iassign(led_state, 3);;
        }
        else if(led_state == 3) 
        {
            iassign(led_state, 4);;
        }
        else if(led_state == 4) 
        {
            iassign(led_state, 5);;
        }
        else if(led_state == 5) 
        {
            iassign(led_state, 9);;
        }
        else if(led_state == 6) 
        {
            iassign(led_state, 10);
        }
        else
        {
            iassign(led_state, 0);
        }
        
        send(157,status);
        if( led_state == 0 )
        {
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Force LED End Successful";
            }
            else
            {
                strPrompt = "|en|Force LED End NOT Successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
        else
        {
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Force LED Successful";
            }
            else
            {
                strPrompt = "|en|Force LED NOT Successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_FORCE_LED
{
    NUMBER 157;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
            led_state
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}
/*------------------- COMMAND 158 Write Fault current (device loop current when it is in fault mode) -----------------*/
COMMAND CMD_WRITE_CONFIG_MA
{
    NUMBER 158;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            InhibitCurrent, WarningCurrent, FaultCurrent, OverrangeCurrent
        }
        REPLY
        {
            response_code, device_status,
            InhibitCurrent, WarningCurrent, FaultCurrent, OverrangeCurrent
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Value is too High or Low";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 159 Read Fault, Inhibit, warning and over range current (device loop current when it is in respective mode) -----------------*/
VARIABLE FaultCurrent
{
    LABEL "Fault Level";
    HELP  "|en|Fault level current in mA.\nRange 0 - 3.6mA";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    CONSTANT_UNIT "|en|mA";
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE 1.0;
}

VARIABLE InhibitCurrent
{
    LABEL "Inhibit Level";
    HELP  "|en|Inhibit level current value in mA.\nRange 1 - 3.6mA";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    CONSTANT_UNIT "|en|mA";
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    DEFAULT_VALUE 2.0;
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

VARIABLE WarningCurrent
{
    LABEL "Warning Level";
    HELP  "|en|Warning level current in mA.\nRange 1 - 4mA";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    CONSTANT_UNIT "|en|mA";
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE 3.0;
}

VARIABLE OverrangeCurrent
{
    LABEL "Over range Level";
    HELP  "|en|Overrange level current in mA.\nRange 20 - 22mA";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    CONSTANT_UNIT "|en|mA";
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
        EDIT_FORMAT ".2f";
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE 21.0;
}

COMMAND CMD_READ_CONFIG_MA
{
    NUMBER 159;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            InhibitCurrent, WarningCurrent, FaultCurrent, OverrangeCurrent
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 160 Configure Alarm Latch ----------------*/
VARIABLE alarm_latch
{
    LABEL "Alarm Latch";
    HELP  "|en|Alarm latch to enable or disable";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE ENUMERATED
    {
        {   0, "|en|Enable"}, 
        {   1, "|en|Disable"}
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_config_latch
{
    LABEL "Alarm Latch";
    HELP  "|en|Configure Alarm to enable or disable";
    DEFINITION 
    {
        DD_STRING strPrompt;
        int result;
        char status[STATUS_SIZE];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(129,status);
        send(180,status);
        if( Current_OpMode != 2 )
        {
            ACKNOWLEDGE("Please move the detector to inhibit mode and try!");
            process_abort();
        }
        else if(Inhibit_Source_Rd == 1)
        {
            ACKNOWLEDGE("Please check if the detector was put to inhibit by the Mobile app.\nThe detector is expected to be set into inhibit mode by HART.");
            process_abort();
        }
        else
        {
            alarm_latch = SELECT_FROM_LIST("Select Alarm Status to","Enable;Disable") ;
            result = send(160,status);
            if(status[STATUS_RESPONSE_CODE] == 0) 
            {   
                strPrompt = "|en|Configure Alarm state Successful";
            }
            else
            {
                strPrompt = "|en|Configure Alarm state NOT successful";
            }
            ACKNOWLEDGE(strPrompt);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_WRITE_CONFIG_ALARM_LATCH
{
    NUMBER 160;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            alarm_latch
        }
        REPLY
        {
            response_code, device_status, alarm_latch
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 161 mA Force----------------*/ 

/* Method to test the functioning of the 4 - 20 mA current loop */
METHOD mth_mA_force
{
    LABEL "|en|Force Loop";
    HELP  "|en|Facility to apply force on 4-20mA loop";
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int use_abort_method;
        int choice;
        int slen1;
        int slen2;
        
        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(7,status);
        if( loop_current_mode == 0 )
        {
            ACKNOWLEDGE ("Device in multidrop mode.This test can not be performed.");
            process_abort();
        }
        
        do 
        {
            get_dictionary_string(choose_analog_output_level, disp_string1, slen1);
            get_dictionary_string(choices_4_20_other_end, disp_string2, slen2);

            /* Dispays to the user to select from 4mA, 20mA, "Other" and "End" options */
            choice = SELECT_FROM_LIST(disp_string1, disp_string2);
        
            switch (choice)
            {
                /* If the user selects 4mA */
                case 0:
                    assign_float(PV.DAQ.ANALOG_VALUE,4.0);
                    break;
                /* If the user selects 20mA */
                case 1:
                    assign_float(PV.DAQ.ANALOG_VALUE,20.0);
                    break;
                /* If the user selects a mA level under "Other" option */
                case 2:
                    get_dictionary_string(loop_output, disp_string1, slen1);
                    GET_DEV_VAR_VALUE(disp_string1, PV.DAQ.ANALOG_VALUE );
                    if( (PV.DAQ.ANALOG_VALUE > 22.5) || (PV.DAQ.ANALOG_VALUE < 0.0) )
                    {
                        ACKNOWLEDGE ("Value out of range");
                        process_abort();
                    }
                    break;
                /* If the user selects the "End" option */
                case 3:
               default:
                    assign_float(PV.DAQ.ANALOG_VALUE,0.0);
                    send(161, status);
                    if (status[STATUS_RESPONSE_CODE] != 0)
                    {
                        display_response_status(161,status[STATUS_RESPONSE_CODE]);
                    }
                    else
                    {
                        ACKNOWLEDGE ("Return to normal operation");
                    }
                    process_abort();
                    break;
            }
            /* Send the command 161 to force mA of user choice */
            send(161, status);

            /* if user aborts test without forcing the current loop to any value */
            if (!use_abort_method)
            {
                /* If unexpected response from the device */
                if ((status[STATUS_RESPONSE_CODE] != 0)
                                && ( status[STATUS_RESPONSE_CODE] != 3)
                                && ( status[STATUS_RESPONSE_CODE] != 4))
                {
                    display_response_status(161,status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(abort_last_error, disp_string1, slen1);
                    DELAY(2, disp_string1);
                    process_abort();
                }
        
                if (status[STATUS_RESPONSE_CODE] == 0)
                {
                    //remove_abort_method(return_to_normal);
                    add_abort_method(loop_release_from_force);
                    use_abort_method = 1;
                }
            }
        
            if (status[STATUS_RESPONSE_CODE])
            {
                 /* 
                  * Display the resonse code from the error 
                  */
                display_response_status(161, status[STATUS_RESPONSE_CODE]);
        
                /* 
                 * If the response code is either 3 or 4, meaning the value
                 * was either too large or too small, do not abort method.
                 * Allow the user to try again. 
                 */ 
                if ((status[STATUS_RESPONSE_CODE] != 3)
                               && ( status[STATUS_RESPONSE_CODE] != 4))
                {
                    display_response_status(161, status[STATUS_RESPONSE_CODE]);
                    disp_string1 = "NOTE: Detector will return to normal operation";
                    ACKNOWLEDGE(disp_string1);
                    remove_abort_method(user_return_to_normal);
                    process_abort();
                }
            }
            else
            {
                var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);
                get_dictionary_string(fld_dev_otpt_fixed, disp_string1, slen1);
                /* Display the forced value to the user */
                acknowledge(disp_string1, var_id);
            }
        } while (choice < 3);
    }
}

COMMAND CMD_FORCE_MALOOP
{
    NUMBER 161;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            PV.DAQ.ANALOG_VALUE
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 162 & 163  Read and write Loop operating mode----------------*/
VARIABLE Loop_OpMode
{
    LABEL "Operating Mode";
    HELP "Loop Specific Operating Mode which can contain any one of the following status\n\n"
                          "Discrete - mA output changes only on pre alarm, alarm and faulty conditions \n"
                          "Proportional - mA output changes proportional to gas leak and faulty conditions\n";
    CLASS DEVICE & DYNAMIC;
    HANDLING WRITE;
    TYPE ENUMERATED
    {
        {0,"|en|Discrete"}
        {1,"|en|Proportional"}
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
}

VARIABLE Loop_OpMode_Rd
{
    LABEL "Operating Mode";
    HELP "Loop Specific Operating Mode which can contain any one of the following status\n\n"
                          "Discrete - mA output changes only on pre alarm, alarm and faulty conditions \n"
                          "Proportional - mA output changes proportional to gas leak and faulty conditions\n";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED
    {
        {0,"|en|Discrete"}
        {1,"|en|Proportional"}
    }
}

METHOD mth_OpMode_chng
{
    LABEL "Operating Mode";
    HELP  "Loop Specific Operating Mode which can contain any one of the following status\n\n"
                          "Discrete - mA output changes only on pre alarm, alarm and faulty conditions \n"
                          "Proportional - mA output changes proportional to gas leak and faulty conditions\n";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        DD_STRING strPrompt;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        mth_inhibitinfo();
        send(162,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   
            strPrompt = "|en|Clear Latches Successful";
        }
        else
        {
           
            strPrompt = "|en|Clear Latches NOT Successful";
        }
        ACKNOWLEDGE(strPrompt);
    }
    VALIDITY IF (login_level > LEVEL0){TRUE;}ELSE{FALSE;}
}


COMMAND CMD_LOOP_WRITE_OPMODE
{
    NUMBER 162;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            Loop_OpMode
        }
        REPLY
        {
            response_code, device_status, Loop_OpMode
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_LOOP_OPMODE
{
    NUMBER 163;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, Loop_OpMode_Rd
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 164 Configure LED brightness level -----------------*/
VARIABLE LEDIntensity
{
    LABEL "LED Intensity";
    HELP  "|en|Represents LED intensity to be any of the following \n\n"
          "|en|Dim"
          "|en|Low"
          "|en|Medium"
          "|en|High"
          "|en|Bright";
    CLASS DEVICE & DYNAMIC;
    HANDLING WRITE;
    TYPE ENUMERATED 
    {
        {1, "Dim"     },
	    {2, "Low"     },
        {3, "Medium"  },
        {4, "High"    },
        {5, "Bright"  }
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE 3;
}

VARIABLE LEDIntensity_Rd
{
    LABEL "LED Intensity";
    HELP  "|en|Represents LED intensity to be any of the following \n\n"
          "|en|Dim"
          "|en|Low"
          "|en|Medium"
          "|en|High"
          "|en|Bright";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED 
    {
        {1, "Dim"     },
        {2, "Low"     },
        {3, "Medium"  },
        {4, "High"    },
        {5, "Bright"  }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE 3;
}

COMMAND CMD_WRITE_LEDINTENSITY
{
    NUMBER 164;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            LEDIntensity
        }
        REPLY
        {
            response_code, device_status,
            LEDIntensity
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Value is too High or Low";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_READ_LEDINTENSITY
{
    NUMBER 165;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            LEDIntensity_Rd
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Value is too High or Low";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 166 Configure LED Mode -----------------*/
VARIABLE LEDMode
{
    LABEL "LED Mode";
    HELP  "|en|Mode can be any of the following\n\n"
          "|en|Normal ON"
          "|en|Alerts"
          "|en|All Off";
    CLASS DEVICE & DYNAMIC;
    HANDLING WRITE;
    TYPE ENUMERATED 
    {
        {0, "Normal ON" },
        {1, "Alerts"     },
        {2, "All Off"   }
    }
    PRE_EDIT_ACTIONS
    {
        mth_inhibitinfo
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE 0;
}

VARIABLE LEDMode_Rd
{
    LABEL "LED Mode";
    HELP  "|en|Mode can be any of the following\n\n"
          "|en|Normal ON"
          "|en|Alerts"
          "|en|All Off";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED 
    {
        {0, "Normal ON" },
        {1, "Alerts"     },
        {2, "All Off"   }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
    DEFAULT_VALUE 0;
}

COMMAND CMD_WRITE_LEDMODE
{
    NUMBER 166;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            LEDMode
        }
        REPLY
        {
            response_code, device_status,
            LEDMode
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Value is too High or Low";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_READ_LEDMODE
{
    NUMBER 167;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            LEDMode_Rd
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Value is too High or Low";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 168 Configure BlueTooth communication enable -----------------*/
VARIABLE BT_Status
{
    LABEL "Bluetooth";
    HELP  "|en|Bluetooth Communication status";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE ENUMERATED 
    {
        {0, "Off"},
	    {1, "On" }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_WRITE_BTSTATUS
{
    NUMBER 168;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            BT_Status
        }
        REPLY
        {
            response_code, device_status,
            BT_Status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_READ_BTSTATUS
{
    NUMBER 169;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            BT_Status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 170 Configure BlueTooth Device Name -----------------*/
VARIABLE BT_DeviceName
{
    LABEL "Detector Id";
    HELP  "|en|Represents the detector name for identification";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ASCII(25);
    POST_EDIT_ACTIONS
    {
       mth_BT_deviceName
    }
}

VARIABLE BT_DeviceName_Rd
{
    LABEL "Detector Id";
    HELP  "|en|Represents the detector name for identification";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ASCII(26);
}

VARIABLE BT_Pin_host
{
    LABEL "BT Pin from host";
    HELP  "|en|BT Pin from host for changing configurable parameters";
    CLASS LOCAL;
    TYPE PASSWORD(4);
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_BT_deviceName
{
    LABEL "Change BT Device Name";
    HELP  "|en|Change Bluetooth Device Name";
    DEFINITION 
    {
        char strPrompt[60];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        strPrompt = "Enter Bluetooth ACCESS PIN";
        GET_DEV_VAR_VALUE(strPrompt, BT_Pin_host);
        save_values();
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_WRITE_BTDEVNAME
{
    NUMBER 170;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            BT_Pin_host, BT_DeviceName,00
        }
        REPLY
        {
            response_code, device_status,
            BT_DeviceName,00
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, "|en|Invalid Device name";
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_READ_BTDEVNAME
{
    NUMBER 171;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            BT_DeviceName_Rd
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 172 Configure BlueTooth PIN -----------------*/
VARIABLE BT_Pin
{
    LABEL "Bluetooth PIN";
    HELP  "|en|Bluetooth PIN";
    CLASS LOCAL;
    TYPE PASSWORD(4);
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

VARIABLE BT_New_Pin
{
    LABEL "Bluetooth PIN";
    HELP  "|en|Bluetooth PIN";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE ASCII(4);
    DEFAULT_VALUE " ";
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_WRITE_BTPIN
{
    NUMBER 172;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            BT_Pin,BT_New_Pin
        }
        REPLY
        {
            response_code, device_status,
            BT_New_Pin
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, "|en|No characters allowed in New PIN";
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

METHOD mth_BT_newPIN
{
    LABEL "Change PIN";
    HELP  "|en|Change Bluetooth PIN";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char strPrompt[60];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        strPrompt = "Enter Current Bluetooth PIN";
        GET_DEV_VAR_VALUE(strPrompt, BT_Pin);
        strPrompt = "Enter NEW Bluetooth PIN";
        GET_DEV_VAR_VALUE(strPrompt, BT_New_Pin);
        send(172,status);
        if(status[STATUS_RESPONSE_CODE] == 0)
        {
            strPrompt = "PIN changed successfully";
            ACKNOWLEDGE(strPrompt);
        }
        else
        { 
            display_response_status(172, status[STATUS_RESPONSE_CODE]);
        }
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

/*------------------- COMMAND 173 Configure BlueTooth Device Address -----------------*/
VARIABLE BT_Address
{
    LABEL "Address";
    HELP  "|en|Bluetooth device address";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    #if __TOKVER__ > 635
    TYPE UNSIGNED_INTEGER(6);
    #endif
    #if __TOKVER__ <= 635
    TYPE UNSIGNED_INTEGER(4);
    #endif
    POST_EDIT_ACTIONS
    {
        mth_BT_deviceAddr
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}
#if __TOKVER__ <= 635
VARIABLE BT_Address_0
{
    LABEL "Address";
    HELP  "|en|Bluetooth device address";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;   
    TYPE UNSIGNED_INTEGER(2);
   
    POST_EDIT_ACTIONS
    {
        mth_BT_deviceAddr
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}
#endif

METHOD mth_BT_deviceAddr
{
    LABEL "Change BT Device Address";
    HELP  "|en|Change Bluetooth Device Address";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char strPrompt[60];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        strPrompt = "Enter Bluetooth ACCESS PIN";
        GET_DEV_VAR_VALUE(strPrompt, BT_Pin_host);
        save_values();
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

#if __TOKVER__ > 635
COMMAND CMD_WRITE_BTADDRESS
{
    NUMBER 173;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            BT_Pin_host, BT_Address
        }
        REPLY
        {
            response_code, device_status,
            BT_Address
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_READ_BTADDRESS
{
    NUMBER 174;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            BT_Address
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}
#endif

#if __TOKVER__ <= 635
COMMAND CMD_WRITE_BTADDRESS
{
    NUMBER 173;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            BT_Pin_host, BT_Address_0, BT_Address
        }
        REPLY
        {
            response_code, device_status,BT_Address_0,
            BT_Address
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_READ_BTADDRESS
{
    NUMBER 174;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,BT_Address_0,
            BT_Address
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}
#endif
/*------------------- COMMAND 175 Configure BlueTooth Device Adv Interval -----------------*/
VARIABLE BT_AdvIntval
{
    LABEL "Adv Interval";
    HELP  "|en|Bluetooth advertise interval";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER(2);
    CONSTANT_UNIT "|en|Sec";
    POST_EDIT_ACTIONS
    {
        mth_BT_deviceIntval
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_BT_deviceIntval
{
    LABEL "Change BT Device Interval";
    HELP  "|en|Change Bluetooth Device advertise Interval";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char strPrompt[60];

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        strPrompt = "Enter Bluetooth ACCESS PIN";
        GET_DEV_VAR_VALUE(strPrompt, BT_Pin_host);
        save_values();
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_WRITE_BTADVINTVL
{
    NUMBER 175;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            BT_Pin_host, BT_AdvIntval
        }
        REPLY
        {
            response_code, device_status,
            BT_AdvIntval
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_READ_BTADVINTVL
{
    NUMBER 176;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            BT_AdvIntval
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Invalid BT ACCESS PIN";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 177 Start/End Inihibit current -----------------*/
VARIABLE inhibit_onoff
{
    LABEL "Inhibit";
    HELP  "";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE inhibit_mode
{
    LABEL "Inhibit";
    HELP  "Indiactes Inhibit ON or OFF";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED 
    {
        {0,"|en|ON"},
        {1,"|en|OFF" }
    }
}

METHOD mth_start_inhibit
{
    LABEL "Start Inhibit";
    HELP "Puts the device in inhibit mode";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        DD_STRING strPrompt;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        
        iassign(inhibit_onoff, 0) ;
        ACKNOWLEDGE("|en|Press OK to confirm \n\nIn this mode the detector will no longer trigger outputs on any events");
        send(177,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   
            strPrompt = "|en|Start Inhibit Successful";
        }
        else
        {           
            strPrompt = "|en|Start Inhibit NOT Successful";
        }
        ACKNOWLEDGE(strPrompt); 
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_end_inhibit
{
    LABEL "End Inhibit";
    HELP "Puts the device into active mode from inhibit mode";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        DD_STRING strPrompt;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        
        iassign(inhibit_onoff, 1);
        ACKNOWLEDGE("|en|Press OK to confirm \n\nAfter exiting this mode the detector will trigger outputs on all events");
        send(177,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   
            strPrompt = "|en|End Inhibit Successful";
        }
        else
        {           
            strPrompt = "|en|End Inhibit NOT Successful";
        }
        ACKNOWLEDGE(strPrompt); 
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

METHOD mth_wr_inhibit
{
    LABEL "Inhibit";
    HELP "|en|Facility to Edit Inhibit Mode";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        DD_STRING strPrompt;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        
        iassign(inhibit_onoff, SELECT_FROM_LIST("Inhibit ON or OFF","Start Inhibit;End Inhibit") );
        if(inhibit_onoff == 0 )
        {
            ACKNOWLEDGE("|en|Press OK to confirm \n\nIn this mode the detector will no longer trigger outputs on any events");
        }
        else
        {
            ACKNOWLEDGE("|en|Press OK to confirm \n\nAfter exiting this mode the detector will trigger outputs on all events");
        }
        send(177,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   if(inhibit_onoff == 0 )
            {
                strPrompt = "|en|Start Inhibit Successful";
            }
            else
            {
                strPrompt = "|en|End Inhibit Successful";
            }
        }
        else
        {
            if(inhibit_onoff == 0 )
            {
                strPrompt = "|en|Start Inhibit NOT Successful";
            }
            else
            {
                strPrompt = "|en|End Inhibit NOT Successful";
            }
        }
        ACKNOWLEDGE(strPrompt); 
    }
    VALIDITY IF(login_level > LEVEL0){TRUE;}ELSE {FALSE;}
}

COMMAND CMD_INHIBIT_ON_OFF
{
    NUMBER 177;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            inhibit_onoff
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       "|en|Device in multidrop mode";
        32, MODE_ERROR,       [busy];
    }
}

COMMAND CMD_INHIBIT_MODE
{
    NUMBER 141;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {

        }
        REPLY
        {
            response_code, device_status, inhibit_mode
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 178 Clear latched faults and warnings -----------------*/
VARIABLE ClrAlarmFaults
{
    LABEL "Clear Alarm";
    HELP  "";
    CLASS DEVICE;
    HANDLING  WRITE;
    VALIDITY IF (login_level > LEVEL0){TRUE;}ELSE{FALSE;}
    TYPE UNSIGNED_INTEGER;
}

METHOD mth_ClrFlts
{
    LABEL "Clear Latches";
    HELP  "Clears all latched states in the device";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        DD_STRING strPrompt;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        iassign(ClrAlarmFaults, 7);
        send(178,status);
        if(status[STATUS_RESPONSE_CODE] == 0) 
        {   
            strPrompt = "|en|Clear Latches Successful";
        }
        else
        {
           
            strPrompt = "|en|Clear Latches NOT Successful";
        }
        ACKNOWLEDGE(strPrompt);
    }
    VALIDITY IF (login_level > LEVEL0){TRUE;}ELSE{FALSE;}
}

COMMAND CMD_CLR_LATCHEDALARMS
{
    NUMBER 178;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
           ClrAlarmFaults
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Command not Processed Succesfully";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 179 Read alarm latch status -----------------*/
VARIABLE alarm_latch_st
{
    LABEL "Alarm Latch";
    HELP  "Alarm latch state either of enable or disable";
    CLASS DEVICE & DYNAMIC;
    HANDLING  READ;
    VALIDITY IF (login_level > LEVEL0){TRUE;}ELSE{FALSE;}
    TYPE ENUMERATED
    {
        {   0, "|en|Enabled"}, 
        {   1, "|en|Disabled"}
    }
}

COMMAND CMD_READ_LATCHEDALARMS
{
    NUMBER 179;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
           
        }
        REPLY
        {
            response_code, device_status, alarm_latch_st
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, "|en|Command not Processed Succesfully";
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/*------------------- COMMAND 180 Read Inhibit Source -----------------*/
VARIABLE Inhibit_Source_Rd
{
    LABEL "Inhibit Source";
    HELP  "|en|Source inhibit has come from\n\n";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(1);
   
    DEFAULT_VALUE 0;
}

COMMAND CMD_READ_INHIBIT_SOURCE
{
    NUMBER 180;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
           
        }
        REPLY
        {
            response_code, device_status, Inhibit_Source_Rd
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        3,  DATA_ENTRY_ERROR, [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR, [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  DATA_ENTRY_ERROR, [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
    }
}

/******************************************************************************************
*                        Miscellaneous functins                                           *
*******************************************************************************************/
/* Displays customized string to user */
METHOD user_return_to_normal
{
    LABEL "user_ret_normal";
    HELP  "";
    DEFINITION 
    {
         DD_STRING disp_string1;

         disp_string1 = "NOTE: Detector will return to normal operation";
         DISPLAY(disp_string1);
         process_abort();
    }
}

/* Method to send release loop from forced mA */
METHOD loop_release_from_force
{
    LABEL "loop_release_from_force";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
    }
}

/* Displays customized string to user */
METHOD user_abort
{
    LABEL "user_abort";
    HELP  "";
    DEFINITION 
    {
         DD_STRING disp_string1;

         disp_string1 = "Abrupted. Operating in progress may be successful";
         DISPLAY(disp_string1);
         //process_abort();
    }
}
